# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from collections import namedtuple
import json
import logging
import flask
from superdesk import get_resource_service
import superdesk
from superdesk.errors import SuperdeskApiError
from superdesk.metadata.item import not_analyzed, ITEM_STATE, PUBLISH_STATES
from superdesk.metadata.utils import aggregations, get_elastic_highlight_query
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.utc import utcnow

from bson.objectid import ObjectId
from eve.utils import ParsedRequest, config
from flask import current_app as app

from apps.archive.archive import SOURCE as ARCHIVE
from apps.archive.common import handle_existing_data, item_schema
from superdesk.publish.publish_queue import PUBLISHED_IN_PACKAGE
from apps.content import push_content_notification
from flask_babel import _

logger = logging.getLogger(__name__)

PUBLISHED = "published"
LAST_PUBLISHED_VERSION = "last_published_version"
QUEUE_STATE = "queue_state"
ERROR_MESSAGE = "error_message"
queue_states = ["pending", "in_progress", "queued", "queued_not_transmitted", "error"]
PUBLISH_STATE = namedtuple("PUBLISH_STATE", ["PENDING", "IN_PROGRESS", "QUEUED", "QUEUED_NOT_TRANSMITTED", "ERROR"])(
    *queue_states
)

published_item_fields = {
    "item_id": {"type": "string", "mapping": not_analyzed},
    "publish_state": {"type": "string"},
    # last_published_version field is set to true for last published version of the item in the published collection
    # and for the older version is set to false. This field is used to display the last version of the digital copy
    # in the published view.
    LAST_PUBLISHED_VERSION: {"type": "boolean", "default": True},
    QUEUE_STATE: {
        "type": "string",
        "default": "pending",
        "allowed": queue_states,
    },
    ERROR_MESSAGE: {"type": "string"},
    "is_take_item": {  # deprecated
        "type": "boolean",
        "default": False,
    },
    "digital_item_id": {"type": "string"},  # deprecated
    "publish_sequence_no": {"type": "integer", "readonly": True},
    "last_queue_event": {"type": "datetime"},
    # item is moved to legal. True if moved else false.
    "moved_to_legal": {"type": "boolean", "default": False},
    # if item is published as part of a package this will
    # be set to package id so transmitter can handle such
    # items differently
    PUBLISHED_IN_PACKAGE: {
        "type": "string",
    },
    "_current_version": {  # must be set explicitly, there is no versioning
        "type": "integer",
    },
}


def get_content_filter():
    """Filter out content of stages not visible to current user (if any).

    :return:
    """
    user = getattr(flask.g, "user", None)
    if user:
        if "invisible_stages" in user:
            stages = user.get("invisible_stages")
        else:
            stages = get_resource_service("users").get_invisible_stages_ids(user.get("_id"))

        if stages:
            return {"bool": {"must_not": {"terms": {"task.stage": stages}}}}


class PublishedItemResource(Resource):
    datasource = {
        "search_backend": "elastic",
        "aggregations": aggregations,
        "es_highlight": get_elastic_highlight_query,
        "default_sort": [("_updated", -1)],
        "elastic_filter_callback": get_content_filter,
        "projection": {
            "old_version": 0,
            "last_version": 0,
        },
    }

    schema = item_schema(published_item_fields)
    etag_ignore_fields = [config.ID_FIELD, "highlights", "item_id", LAST_PUBLISHED_VERSION, "moved_to_legal"]

    privileges = {"POST": "publish_queue", "PATCH": "publish_queue"}
    item_methods = ["GET", "PATCH"]
    additional_lookup = {"url": 'regex("[\w,.:-]+")', "field": "item_id"}


class PublishedItemService(BaseService):
    """
    PublishedItemService class is the base class for ArchivedService.
    """

    SEQ_KEY_NAME = "published_item_sequence_no"

    def on_fetched(self, docs):
        """
        Overriding this to enhance the published article with the one in archive collection
        """

        self.enhance_with_archive_items(docs[config.ITEMS])

    def on_fetched_item(self, doc):
        """
        Overriding this to enhance the published article with the one in archive collection
        """

        self.enhance_with_archive_items([doc])

    def on_create(self, docs):
        """Runs on create.

        An article can be published multiple times in its lifetime. So, it's necessary to preserve the _id which comes
        from archive collection. Also, sets the expiry on the published item and removes the lock information.
        """

        for doc in docs:
            self.raise_if_not_marked_for_publication(doc)
            doc[config.LAST_UPDATED] = doc[config.DATE_CREATED] = utcnow()
            self.set_defaults(doc)

    def on_update(self, updates, original):
        if ITEM_STATE in updates:
            self.raise_if_not_marked_for_publication(updates)

    def on_updated(self, updates, original):
        if "marked_for_user" in updates:
            updated = original.copy()
            updated.update(updates)
            # Send notification on mark-for-user operation
            get_resource_service("archive").handle_mark_user_notifications(updates, original)
            push_content_notification([updated, original])  # see SDBELGA-192

    def raise_if_not_marked_for_publication(self, doc):
        """
        Item should be one of the PUBLISH_STATES. If not raise error.
        """
        if doc.get(ITEM_STATE) not in PUBLISH_STATES and doc.get(ITEM_STATE) != "spiked":
            raise SuperdeskApiError.badRequestError(
                _("Invalid state ({state}) for the Published item.").format(state=doc.get(ITEM_STATE))
            )

    def set_defaults(self, doc):
        doc["item_id"] = doc[config.ID_FIELD]
        doc["versioncreated"] = utcnow()
        doc["publish_sequence_no"] = get_resource_service("sequences").get_next_sequence_number(self.SEQ_KEY_NAME)
        doc.pop(config.ID_FIELD, None)
        doc.pop("lock_user", None)
        doc.pop("lock_time", None)
        doc.pop("lock_action", None)
        doc.pop("lock_session", None)

    def enhance_with_archive_items(self, items):
        if items:
            ids = list(set([item.get("item_id") for item in items if item.get("item_id")]))
            archive_items = []
            archive_lookup = {}
            if ids:
                query = {"$and": [{config.ID_FIELD: {"$in": ids}}]}
                archive_req = ParsedRequest()
                archive_req.max_results = len(ids)
                # can't access published from elastic due filter on the archive resource hence going to mongo
                archive_items = list(
                    superdesk.get_resource_service(ARCHIVE).get_from_mongo(req=archive_req, lookup=query)
                )

                for item in archive_items:
                    handle_existing_data(item)
                    archive_lookup[item[config.ID_FIELD]] = item

            for item in items:
                archive_item = archive_lookup.get(item.get("item_id"), {config.VERSION: item.get(config.VERSION, 1)})

                updates = {
                    config.ID_FIELD: item.get("item_id"),
                    "item_id": item.get(config.ID_FIELD),
                    "lock_user": archive_item.get("lock_user", None),
                    "lock_time": archive_item.get("lock_time", None),
                    "lock_action": archive_item.get("lock_action", None),
                    "lock_session": archive_item.get("lock_session", None),
                    "archive_item": archive_item if archive_item else None,
                }

                item.update(updates)
                handle_existing_data(item)

    def on_delete(self, doc):
        """Deleting a published item has a workflow which is implemented in remove_expired().

        Overriding to avoid other services from invoking this method accidentally.
        """

        if app.testing:
            super().on_delete(doc)
        else:
            raise NotImplementedError(
                _("Deleting a published item has a workflow which is implemented in remove_expired().")
            )

    def delete_action(self, lookup=None):
        """Deleting a published item has a workflow which is implemented in remove_expired().

        Overriding to avoid other services from invoking this method accidentally.
        """

        if app.testing:
            super().delete_action(lookup)
        else:
            raise NotImplementedError(
                _("Deleting a published item has a workflow which is implemented in remove_expired().")
            )

    def on_deleted(self, doc):
        """Deleting a published item has a workflow which is implemented in remove_expired().

        Overriding to avoid other services from invoking this method accidentally.
        """

        if app.testing:
            super().on_deleted(doc)
        else:
            raise NotImplementedError(
                _("Deleting a published item has a workflow which is implemented in remove_expired().")
            )

    def get_other_published_items(self, _id):
        try:
            query = {"query": {"filtered": {"filter": {"term": {"item_id": _id}}}}}
            request = ParsedRequest()
            request.args = {"source": json.dumps(query)}
            return super().get(req=request, lookup=None)
        except Exception:
            return []

    def get_last_published_version(self, _id):
        """Returns the last published entry for the passed item id

        :param _id:
        :return:
        """
        try:
            query = {
                "query": {
                    "filtered": {
                        "filter": {
                            "bool": {"must": [{"term": {"item_id": _id}}, {"term": {LAST_PUBLISHED_VERSION: True}}]}
                        }
                    }
                }
            }

            request = ParsedRequest()
            request.args = {"source": json.dumps(query), "repo": "published"}
            items = list(self.get(req=request, lookup=None))
            if items:
                return items[0]
        except Exception:
            return None

    def get_rewritten_items_by_event_story(self, event_id, rewrite_id, rewrite_field):
        """Returns all the rewritten stories from published and archive for a given event and rewrite_id.

        :param str event_id: event id of the document
        :param str rewrite_id: rewrite id of the document
        :param str rewrite_field: field name 'rewrite_of' or 'rewritten_by'
        """
        try:
            query = {
                "query": {
                    "filtered": {
                        "filter": {
                            "bool": {"must": [{"term": {"event_id": event_id}}, {"term": {rewrite_field: rewrite_id}}]}
                        }
                    }
                }
            }

            request = ParsedRequest()
            request.args = {"source": json.dumps(query), "repo": "archive,published"}
            return list(get_resource_service("search").get(req=request, lookup=None))
        except Exception:
            return []

    def is_rewritten_before(self, item_id):
        """Checks if the published item is rewritten before.

        :param _id: item_id of the published item
        :return: True is it is rewritten before
        """
        doc = self.find_one(req=None, item_id=item_id)
        return doc and "rewritten_by" in doc and doc["rewritten_by"]

    def update_published_items(self, _id, field, state):
        items = self.get_other_published_items(_id)
        for item in items:
            try:
                super().system_update(ObjectId(item[config.ID_FIELD]), {field: state}, item)
            except Exception:
                # This part is used in unit testing
                super().system_update(item[config.ID_FIELD], {field: state}, item)

    def delete_by_article_id(self, _id):
        """Removes the article from the published collection.

        Removes published queue entries.

        :param str _id: id of the document to be deleted. In mongo, it is the item_id
        """
        lookup = {"item_id": _id}
        self.delete(lookup=lookup)
        get_resource_service("publish_queue").delete_by_article_id(_id)

    def find_one(self, req, **lookup):
        item = super().find_one(req, **lookup)
        handle_existing_data(item)

        return item

    def move_to_archived(self, _id):
        published_items = list(self.get_from_mongo(req=None, lookup={"item_id": _id}))
        if not published_items:
            return
        get_resource_service("archived").post(published_items)
        self.delete_by_article_id(_id)

    def set_moved_to_legal(self, item_id, version, status):
        """Update the legal flag.

        :param str item_id: id of the document
        :param int version: version of the document
        :param boolean status: True if the item is moved to legal else false
        """
        items = self.get_other_published_items(item_id)

        for item in items:
            try:
                if item.get(config.VERSION) <= version and not item.get("moved_to_legal", False):
                    super().system_update(ObjectId(item.get(config.ID_FIELD)), {"moved_to_legal": status}, item)
            except Exception:
                logger.exception(
                    "Failed to set the moved_to_legal flag " "for item {} and version {}".format(item_id, version)
                )

    def get_published_items_by_moved_to_legal(self, item_ids, move_to_legal):
        """Get the pulished items where flag is moved.

        :param list item_ids: List of item
        :param bool move_to_legal: move_to_legal boolean flag
        :return: list of published items
        """
        if item_ids:
            try:
                query = {
                    "query": {
                        "filtered": {
                            "filter": {
                                "and": [{"terms": {"item_id": item_ids}}, {"term": {"moved_to_legal": move_to_legal}}]
                            }
                        }
                    }
                }

                request = ParsedRequest()
                request.args = {"source": json.dumps(query)}
                return list(super().get(req=request, lookup=None))
            except Exception:
                logger.exception(
                    "Failed to get published items " "by moved to legal: {} -- ids: {}.".format(move_to_legal, item_ids)
                )

        return []



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from .service import SearchService
from .resource import SearchResource


def init_app(app):
    endpoint_name = "search_capi"
    service = SearchService(endpoint_name, backend=superdesk.get_backend())
    SearchResource(endpoint_name, app=app, service=service)
    superdesk.privilege(name=endpoint_name, label="Content API Search", description="Content API Search")



# This file is part of Superdesk.
#
# Copyright 2020 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import imaplib
from os.path import join
import time
import logging
from flask_babel import lazy_gettext as l_
import superdesk
from superdesk.auth import oauth
from superdesk.errors import IngestEmailError
from superdesk.io.registry import register_feeding_service, register_feeding_service_parser
from . import EmailFeedingService


logger = logging.getLogger(__name__)


class GMailFeedingService(EmailFeedingService):
    """
    Feeding Service class which can read the article(s) from a configured mail box.
    """

    NAME = "gmail"

    ERRORS = [
        IngestEmailError.emailError().get_error_description(),
        IngestEmailError.emailLoginError().get_error_description(),
    ]

    label = "Gmail"

    fields = [
        {
            "type": "url_request",
            "label": l_("Log-in with GMail"),
        },
        {
            "id": "email",
            "type": "string",
            "label": l_("email"),
            "readonly": True,
            "placeholder": l_("This field will be automatically filled once you've logged using log-in button above"),
        },
        {
            "id": "mailbox",
            "type": "text",
            "label": l_("Mailbox"),
            "default_value": "INBOX",
            "placeholder": l_("Mailbox"),
            "required": True,
            "errors": {6004: "Authentication error."},
        },
        {"id": "filter", "type": "text", "label": l_("Filter"), "placeholder": "Filter", "required": False},
    ]

    @classmethod
    def init_app(cls, app):
        # we need to access config to set the URL, so we do it here
        field = next(f for f in cls.fields if f["type"] == "url_request")
        field["url"] = join(app.config["SERVER_URL"], "login", "google", "{URL_ID}")

    def _test(self, provider):
        self._update(provider, update=None, test=True)

    def authenticate(self, provider: dict, config: dict) -> imaplib.IMAP4_SSL:
        oauth2_token_service = superdesk.get_resource_service("oauth2_token")
        token = oauth2_token_service.find_one(req=None, _id=provider["url_id"])
        if token is None:
            raise IngestEmailError.notConfiguredError(ValueError(l_("You need to log in first")), provider=provider)
        imap = imaplib.IMAP4_SSL("imap.gmail.com")

        if token["expires_at"].timestamp() < time.time() + 600:
            logger.info("Refreshing token for {provider_name}".format(provider_name=provider["name"]))
            token = oauth.refresh_google_token(token["_id"])

        auth_string = "user={email}\x01auth=Bearer {token}\x01\x01".format(
            email=token["email"], token=token["access_token"]
        )
        imap.authenticate("XOAUTH2", lambda __: auth_string.encode())
        return imap


register_feeding_service(GMailFeedingService)
register_feeding_service_parser(GMailFeedingService.NAME, "email_rfc822")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from operator import itemgetter
from eve.render import send_response
from flask import Blueprint, current_app as app
from superdesk.utc import utcnow

bp = Blueprint("locators", __name__)


class LocatorIndex:
    """Locator index."""

    def __init__(self):
        self.locators = []

    def register(self, locators):
        """Register subjects.

        :param locators: list with locators
        """
        self.locators = locators

    def find_cities(self, country_code=None, state_code=None):
        """
        Fetches cities in the locators.json file.

        Returns all the cities in the locators.json file.
        When country_code is passed it gets cities in the country identified by country_code.

        :param  country_code: Country Code as defined in the locators.json file.
                For example, AU for Australia, NZ for New Zealand.
        :param state_code: State Code as defined in the locators.json file.
               For example, VIC for Victoria, NV for Nevada.
        :return:
            Returns Cities in a State when both country_code and state_code are passed.
            Returns Cities in a Country when only country_code is passed.
            Returns Cities across all countries when nothing is passed.
        """

        if country_code and state_code:
            cities = [
                {
                    "city_code": city["qcode"],
                    "city": city["name"],
                    "alt_name": city.get("alt_name", ""),
                    "tz": city["timezone"],
                    "dateline": city["dateline_format"],
                    "state": state["name"],
                    "state_code": state["qcode"],
                    "country": country["name"],
                    "country_code": country["qcode"],
                }
                for country in self.locators
                if country["qcode"] == country_code
                for state in country["states"]
                if state["qcode"] == state_code
                for city in state["cities"]
            ]
            cities.sort(key=itemgetter("city"))
        elif country_code:
            cities = [
                {
                    "city_code": city["qcode"],
                    "city": city["name"],
                    "alt_name": city.get("alt_name", ""),
                    "tz": city["timezone"],
                    "dateline": city["dateline_format"],
                    "state": state["name"],
                    "state_code": state["qcode"],
                    "country": country["name"],
                    "country_code": country["qcode"],
                }
                for country in self.locators
                if country["qcode"] == country_code
                for state in country["states"]
                for city in state["cities"]
            ]
            cities.sort(key=itemgetter("state", "city"))
        else:
            cities = [
                {
                    "city_code": city["qcode"],
                    "city": city["name"],
                    "alt_name": city.get("alt_name", ""),
                    "tz": city["timezone"],
                    "dateline": city["dateline_format"],
                    "state": state["name"],
                    "state_code": state["qcode"],
                    "country": country["name"],
                    "country_code": country["qcode"],
                }
                for country in self.locators
                for state in country["states"]
                for city in state["cities"]
            ]
            cities.sort(key=itemgetter("country", "state", "city"))

        return cities


@bp.route("/cities/", methods=["GET", "OPTIONS"])
@bp.route("/country/<country_code>", methods=["GET", "OPTIONS"])
@bp.route("/country/<country_code>/state/<state_code>", methods=["GET", "OPTIONS"])
def get_cities(country_code=None, state_code=None):
    """
    Fetches cities and sends the list as response body.

    :param  country_code: Country Code as defined in the locators.json file.
            For example, AU for Australia, NZ for New Zealand.
    :param state_code: State Code as defined in the locators.json file. For example, VIC for Victoria, NV for Nevada.
    :return:
        Returns HTTP Response with body {'_items': cities, '_meta': {'total': City Count}}.
    """

    cities = app.locators.find_cities(country_code=country_code, state_code=state_code)

    if cities and len(cities):
        response_data = {"_items": cities, "_meta": {"total": len(cities)}}
        return send_response(None, (response_data, utcnow(), None, 200))
    else:
        return send_response(None, ({}, utcnow(), None, 404))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import re
from apps.content_filters.filter_condition.filter_condition_operator import (
    FilterConditionOperatorsEnum,
    ComparisonOperator,
    ExistsOperator,
)


class FilterConditionValue:

    mongo_mapper = {
        FilterConditionOperatorsEnum.startswith: "^{}",
        FilterConditionOperatorsEnum.like: ".*{}.*",
        FilterConditionOperatorsEnum.notlike: ".*{}.*",
        FilterConditionOperatorsEnum.endswith: ".*{}",
    }

    elastic_mapper = {
        FilterConditionOperatorsEnum.startswith: "{}:{}*",
        FilterConditionOperatorsEnum.like: "{}:*{}*",
        FilterConditionOperatorsEnum.notlike: "{}:*{}*",
        FilterConditionOperatorsEnum.endswith: "{}:*{}",
        FilterConditionOperatorsEnum.match: "{}:{}",
    }

    def __init__(self, operator, value):
        self.operator = operator
        self.value = value
        self.mongo_regex = self.mongo_mapper.get(operator.operator)
        self.elastic_regex = self.elastic_mapper.get(operator.operator)

    def get_value(self, field, operator):
        if isinstance(operator, ComparisonOperator) or isinstance(operator, ExistsOperator):
            t = field.get_type()
            if t is bool:
                return self.value.lower() in ("yes", "true", "t", "1")
            return t(self.value)
        else:
            return self.get_mongo_value(field)

    def get_mongo_value(self, field):
        if self.mongo_regex:
            return self._get_regex_value()
        else:
            return self._get_value(field)

    def get_elastic_value(self, field, operator):
        if isinstance(operator, ComparisonOperator):
            t = field.get_type()
            if t is bool:
                return self.value.lower() in ("yes", "true", "t", "1"), field.get_entity_name()
            return t(self.value), field.get_entity_name()

        if self.elastic_regex:
            return self.elastic_regex.format(field.get_entity_name(), self.value), "query"
        else:
            return self._get_value(field), field.get_entity_name()

    def _get_regex_value(self):
        return re.compile(self.mongo_regex.format(self.value), re.IGNORECASE)

    def _get_value(self, field):
        t = field.get_type()
        if self.value.find(",") > 0:
            return [t(x) for x in self.value.strip().split(",")]
        return [t(self.value)]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re

from datetime import datetime

from superdesk.errors import IngestApiError, ParserError
from superdesk.io.registry import register_feeding_service
from superdesk.io.feeding_services.http_base_service import HTTPFeedingServiceBase

utcfromtimestamp = datetime.utcfromtimestamp


class BBCLDRSFeedingService(HTTPFeedingServiceBase):
    """
    Feeding Service class for reading BBC's Local Democracy Reporting Service
    """

    # Following the api spec at https://docs.ldrs.org.uk/

    NAME = "bbc_ldrs"
    ERRORS = [ParserError.parseMessageError().get_error_description()]

    label = "BBC Local Democracy Reporter Service"

    fields = [
        {
            "id": "url",
            "type": "text",
            "label": "LDRS URL",
            "placeholder": "LDRS URL",
            "required": True,
            "default": "https://api.ldrs.org.uk/v1/item",
        },
        {
            "id": "api_key",
            "type": "text",
            "label": "API Key",
            "placeholder": "API Key",
            "required": True,
            "default": "",
        },
    ]
    HTTP_AUTH = False

    def __init__(self):
        super().__init__()

    def _test(self, provider):
        config = self.config
        url = config["url"]
        api_key = config["api_key"]

        # limit the data to a single article and filter out all article fields
        # to save bandwidth
        params = {"limit": 1, "fields": "id"}
        headers = {"apikey": api_key}

        self.get_url(url, params=params, headers=headers)

    def _update(self, provider, update):
        json_items = self._fetch_data()
        parsed_items = []

        for item in json_items:
            try:
                parser = self.get_feed_parser(provider, item)
                parsed_items.append(parser.parse(item))
            except Exception as ex:
                raise ParserError.parseMessageError(ex, provider, data=item)

        return parsed_items

    def _fetch_data(self):
        url = self.config["url"]
        api_key = self.config["api_key"]

        last_update = self.provider.get("last_updated", utcfromtimestamp(0)).strftime("%Y-%m-%dT%H:%M:%S")

        # Results are pagified so we'll read this many at a time
        offset_jump = 10

        params = {"start": last_update, "limit": offset_jump}
        headers = {"apikey": api_key}

        items = []

        offset = 0
        while True:
            params["offset"] = offset

            response = self.get_url(url, params=params, headers=headers)
            # The total number of results are given to us in json, get them
            # via a regex to read the field so we don't have to convert the
            # whole thing to json pointlessly
            item_ident = re.search('"total": *[0-9]*', response.text).group()
            results_str = re.search("[0-9]+", item_ident).group()

            if results_str is None:
                raise IngestApiError.apiGeneralError(Exception(response.text), self.provider)

            num_results = int(results_str)

            if num_results > 0:
                items.append(response.text)

            if offset >= num_results:
                return items

            offset += offset_jump

        return items


register_feeding_service(BBCLDRSFeedingService)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""
Superdesk OpenID Connect Authentication

.. versionadded:: 2.1

Configuration
~~~~~~~~~~~~~

Superdesk supports OIDC authentication via Keycloak. To enable OIDC auth module, set environment variable
``OIDC_ENABLED`` to True.
Setting :ref:`settings.secret_key` is also required.

Next you need to create a server client (with **confidential**
`access type <https://www.keycloak.org/docs/latest/server_admin/#_access-type>`_) and web client (with **public**
access type). Both with ``Valid Redirect URIs`` set to :ref:`settings.default.client_url`.

Finally with your Keycloak server is running at ``localhost:8080``, ``SUPERDESK``, ``server_client``, ``web_client``
are your keycloak realm, server client, and web client respectively, set following environment variables::

    OIDC_ENABLED=True
    OIDC_ISSUER_URL=http://localhost:8080/auth/realms/SUPERDESK
    OIDC_SERVER_CLIENT=server_client
    OIDC_SERVER_CLIENT_SECRET=server-client-secret
    OIDC_WEB_CLIENT=web-client
"""
import logging

import superdesk

from .auth import OIDCAuthResource, OIDCAuthService

logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = "auth_oidc"
    if app.config["OIDC_ENABLED"] and not app.config["SECRET_KEY"]:
        logger.warn("SECRET_KEY is not set")

    app.client_config["oidc_auth"] = app.config["OIDC_ENABLED"] and app.config["SECRET_KEY"]
    if app.client_config["oidc_auth"]:
        issuer = app.config["OIDC_ISSUER_URL"]
        app.config.setdefault(
            "OIDC_CLIENT_SECRETS",
            {
                "web": {
                    "issuer": issuer,
                    "auth_uri": issuer + "/protocol/openid-connect/auth",
                    "client_id": app.config["OIDC_SERVER_CLIENT"],
                    "client_secret": app.config["OIDC_SERVER_CLIENT_SECRET"],
                    "userinfo_uri": issuer + "/protocol/openid-connect/userinfo",
                    "token_uri": issuer + "/protocol/openid-connect/token",
                    "token_introspection_uri": issuer + "/protocol/openid-connect/token/introspect",
                }
            },
        )
        url, realm = issuer.split("/realms/")
        app.client_config["keycloak_config"] = {
            "url": url,
            "realm": realm,
            "clientId": app.config["OIDC_WEB_CLIENT"],
            "redirectUri": app.config["OIDC_BROWSER_REDIRECT_URL"],
        }
        service = OIDCAuthService("auth", backend=superdesk.get_backend(), app=app)
        OIDCAuthResource(endpoint_name, app=app, service=service)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, CONTENT_STATE
from eve.utils import config
from apps.publish.enqueue.enqueue_service import EnqueueService


class EnqueuePublishedService(EnqueueService):
    def get_subscribers(self, doc, target_media_type):
        """Get the subscribers for this document based on the target_media_type for publishing.

        1. Get all active subscribers.
        2. Filter the subscriber list based on the publish filter and global filters (if configured).
            a. Rewrites are sent to subscribers that received the original item or the previous rewrite.

        :param dict doc: Document to publish/correct/kill
        :param str target_media_type: Valid values are - Wire, Digital.
        :return: (list, dict, dict) List of filtered subscribers, product codes per subscriber,
                associations per subscriber
        """
        subscribers, rewrite_subscribers = [], []
        subscriber_codes, codes, rewrite_codes = {}, {}, {}
        associations, rewrite_associations = {}, {}

        rewrite_of = doc.get("rewrite_of")

        # Step 1
        query = {"is_active": True}
        subscribers = list(get_resource_service("subscribers").get(req=None, lookup=query))

        # Step 2b
        if doc.get(ITEM_TYPE) in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
            if rewrite_of:
                item_ids = [rewrite_of]

                query = {
                    "$and": [
                        {"item_id": {"$in": item_ids}},
                        {"publishing_action": {"$in": [CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED]}},
                    ]
                }

                (
                    rewrite_subscribers,
                    rewrite_codes,
                    rewrite_associations,
                ) = self._get_subscribers_for_previously_sent_items(query)

        # Step 2
        subscribers, codes = self.filter_subscribers(doc, subscribers, target_media_type)

        if rewrite_subscribers:
            subscribers_ids = set(s[config.ID_FIELD] for s in rewrite_subscribers)
            subscribers = rewrite_subscribers + [s for s in subscribers if s[config.ID_FIELD] not in subscribers_ids]

        if rewrite_codes:
            # join the codes
            subscriber_codes.update(rewrite_codes)

        if codes:
            # join the codes
            subscriber_codes.update(codes)

        # update associations
        self._update_associations(associations, rewrite_associations)

        # handle associations
        associations = self._filter_subscribers_for_associations(subscribers, doc, target_media_type, associations)

        return subscribers, subscriber_codes, associations



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging

from flask import current_app as app

import superdesk
from superdesk import get_resource_service, config
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.metadata.item import (
    ITEM_STATE,
    CONTENT_TYPE,
    ITEM_TYPE,
    GUID_FIELD,
    GUID_TAG,
    PUBLISH_STATES,
    CONTENT_STATE,
)
from superdesk.notification import push_notification
from superdesk.services import BaseService
from superdesk.metadata.utils import item_url, generate_guid
from .common import get_user, get_expiry, item_operations, ITEM_OPERATION, set_sign_off, get_auth
from superdesk.workflow import is_workflow_state_transition_valid
from apps.archive.archive import ArchiveResource, SOURCE as ARCHIVE
from apps.archive.common import ITEM_EVENT_ID, ITEM_UNLINK, clear_rewritten_flag
from apps.packages import PackageService
from superdesk.metadata.packages import LINKED_IN_PACKAGES, PACKAGE
from superdesk.utc import get_expiry_date
from apps.item_lock.components.item_lock import push_unlock_notification
from flask_babel import _
from bson.objectid import ObjectId

logger = logging.getLogger(__name__)

EXPIRY = "expiry"
REVERT_STATE = "revert_state"
ITEM_SPIKE = "spike"
ITEM_UNSPIKE = "unspike"
GROUPS = "groups"
DELETED_GROUPS = "deleted_groups"
item_operations.extend([ITEM_SPIKE, ITEM_UNSPIKE])


class ArchiveSpikeResource(ArchiveResource):
    endpoint_name = "archive_spike"
    resource_title = endpoint_name
    datasource = {"source": ARCHIVE}

    url = "archive/spike"
    item_url = item_url

    resource_methods = []
    item_methods = ["PATCH"]

    privileges = {"PATCH": "spike"}


class ArchiveUnspikeResource(ArchiveResource):
    endpoint_name = "archive_unspike"
    resource_title = endpoint_name
    datasource = {"source": ARCHIVE}

    url = "archive/unspike"
    item_url = item_url

    resource_methods = []
    item_methods = ["PATCH"]

    privileges = {"PATCH": "unspike"}


class ArchiveSpikeService(BaseService):
    def on_update(self, updates, original):
        updates[ITEM_OPERATION] = ITEM_SPIKE
        self._validate_item(original)
        self.update_rewrite(original)
        set_sign_off(updates, original=original)

    def _validate_item(self, original):
        """Validates that an item can be deleted.

        Raises an exception if the item is linked in a package, the idea being that you don't want to
        inadvertently remove the item from the packages, this force that to be done as a conscious action.

        :param original:
        :raise: An exception or nothing
        """
        packages = [x.get(PACKAGE) for x in original.get(LINKED_IN_PACKAGES, [])]
        if packages:
            query = {"$and": [{config.ID_FIELD: {"$in": packages}}]}
            cursor = get_resource_service(ARCHIVE).get_from_mongo(req=None, lookup=query)
            if cursor.count() > 0:
                raise SuperdeskApiError.badRequestError(
                    message=_('The item "{slugline}" is in a package').format(slugline=original.get("slugline", ""))
                    + _(" it needs to be removed before the item can be spiked")
                )

    def update_rewrite(self, original):
        """Removes the reference from the rewritten story in published collection."""
        if original.get("rewrite_of") and original.get(ITEM_EVENT_ID):
            clear_rewritten_flag(original.get(ITEM_EVENT_ID), original[config.ID_FIELD], "rewritten_by")

        # write the rewritten_by to the story before spiked
        archive_service = get_resource_service(ARCHIVE)

        if original.get("rewritten_by"):
            # you are spike the story from which the rewrite was triggered.
            # in this case both rewrite_of and rewritten_by are published.
            rewrite_id = original.get("rewritten_by")
            rewritten_by = archive_service.find_one(req=None, _id=rewrite_id)
            archive_service.system_update(rewrite_id, {"rewrite_of": None, "rewrite_sequence": 0}, rewritten_by)
            app.on_archive_item_updated({"rewrite_of": None, "rewrite_sequence": 0}, original, ITEM_UNLINK)

    def _removed_refs_from_package(self, item):
        """Remove reference from the package of the spiked item.

        :param item:
        """
        PackageService().remove_spiked_refs_from_package(item)

    def _get_spike_expiry(self, desk_id, stage_id):
        """Get spike expiry.

        If there is a SPIKE_EXPIRY_MINUTES setting then that is used to set the spike expiry.
        If a None value is configured then the desk/stage value is returned.
        :param desk_id:
        :param stage_id:
        :return:
        """
        # If no maximum spike expiry is set then return the desk/stage values
        if app.settings["SPIKE_EXPIRY_MINUTES"] is None:
            return get_expiry(desk_id=desk_id, stage_id=stage_id)
        else:
            return get_expiry_date(app.settings["SPIKE_EXPIRY_MINUTES"])

    def update(self, id, updates, original):
        original_state = original[ITEM_STATE]
        if not is_workflow_state_transition_valid(ITEM_SPIKE, original_state):
            raise InvalidStateTransitionError()

        archive_service = get_resource_service(ARCHIVE)
        user = get_user(required=True)
        item = archive_service.find_one(req=None, _id=id)
        task = item.get("task", {})

        updates[EXPIRY] = self._get_spike_expiry(desk_id=task.get("desk"), stage_id=task.get("stage"))
        updates[REVERT_STATE] = item.get(ITEM_STATE, None)

        if original.get("rewrite_of"):
            updates["rewrite_of"] = None

        if original.get("rewritten_by"):
            updates["rewritten_by"] = None

        if original.get("broadcast"):
            updates["broadcast"] = None

        if original.get("rewrite_sequence"):
            updates["rewrite_sequence"] = None

        if original.get("marked_for_user"):
            # remove marked_for_user on spike and keep it as previous_marked_user for history
            updates["previous_marked_user"] = original["marked_for_user"]
            updates["marked_for_user"] = None

        published_service = get_resource_service("published")
        published_items = published_service.find({"guid": original.get("guid"), "state": "unpublished"})

        for item in published_items:
            if item:
                item["state"] = CONTENT_STATE.SPIKED
                item["operation"] = ITEM_SPIKE
                published_service.patch(id=item.pop(config.ID_FIELD), updates=item)

        if original.get("translation_id") and original.get("translated_from"):
            # remove translations info from the translated item on spike
            updates["translated_from"] = None
            updates["translation_id"] = None

            id_to_remove = original.get(config.ID_FIELD)

            # Remove the translated item from the list of translations in the original item
            # where orignal item can be in archive or in both archive and published resource as well
            translated_from = archive_service.find_one(req=None, _id=original.get("translated_from"))
            translated_from_id = translated_from.get(config.ID_FIELD)

            self._remove_translations(archive_service, translated_from, id_to_remove)

            if translated_from.get("state") in PUBLISH_STATES:
                published_service = get_resource_service("published")
                published_items = list(
                    published_service.get_from_mongo(req=None, lookup={"item_id": translated_from_id})
                )

                if published_items:
                    for item in published_items:
                        self._remove_translations(published_service, item, id_to_remove)

        # remove any relation with linked items
        updates[ITEM_EVENT_ID] = generate_guid(type=GUID_TAG)

        # remove lock
        updates.update(
            {
                "lock_user": None,
                "lock_session": None,
            }
        )

        if original[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
            # remove links from items in the package
            package_service = PackageService()
            items = package_service.get_item_refs(original)
            for item in items:
                package_item = archive_service.find_one(req=None, _id=item[GUID_FIELD])
                if package_item:
                    linked_in_packages = [
                        linked
                        for linked in package_item.get(LINKED_IN_PACKAGES, [])
                        if linked.get(PACKAGE) != original.get(config.ID_FIELD)
                    ]
                    super().system_update(
                        package_item[config.ID_FIELD], {LINKED_IN_PACKAGES: linked_in_packages}, package_item
                    )

            # keep the structure of old group in order to be able to unspike the package
            updates[DELETED_GROUPS] = original[GROUPS]
            # and remove all the items from the package
            updates["groups"] = []

        item = self.backend.update(self.datasource, id, updates, original)
        push_notification("item:spike", item=str(id), user=str(user.get(config.ID_FIELD)))

        history_updates = dict(updates)
        if original.get("task"):
            history_updates["task"] = original.get("task")
        app.on_archive_item_updated(history_updates, original, ITEM_SPIKE)
        self._removed_refs_from_package(id)
        return item

    def on_updated(self, updates, original):
        get_resource_service("archive_broadcast").spike_item(original)

        if original.get("lock_user"):
            user = get_user()
            auth = get_auth()
            push_unlock_notification(original, user["_id"], auth["_id"])

        if updates.get("previous_marked_user") and not updates.get("marked_for_user"):
            # send notification so that marked for me list can be updated
            get_resource_service("archive").handle_mark_user_notifications(updates, original, False)

    def _remove_translations(self, service, article, id_to_remove):
        """Upadte translation info for the original article in archive or published resource.
        :param service: service for resource endpoint
        :param article: article to be updated
        :param id_to_remove: id of translated item to be removed from the list of translations in original item
        """

        translations = article.get("translations")
        article_id = article.get(config.ID_FIELD)

        if ObjectId.is_valid(article_id) and not isinstance(article_id, ObjectId):
            article_id = ObjectId(article_id)

        if translations:
            translations = [t for t in translations if t != id_to_remove]
            updates = {"translations": translations}
            if not translations:
                updates.update({"translation_id": None})
            service.system_update(article_id, updates, article)


class ArchiveUnspikeService(BaseService):
    def set_unspike_updates(self, doc, updates):
        """Generate changes for a given doc to unspike it.

        :param doc: document to unspike
        :param updates: updates dict
        """
        updates.update(
            {
                REVERT_STATE: None,
                EXPIRY: None,
                ITEM_STATE: doc.get(REVERT_STATE),
                ITEM_OPERATION: ITEM_UNSPIKE,
            }
        )

        task_info = updates.get("task", {})
        desk_id = task_info.get("desk")
        stage_id = task_info.get("stage")

        if not desk_id:  # no desk was set on unspike, restore previous
            desk_id = doc.get("task", {}).get("desk")
            stage_id = None

        if not stage_id and desk_id:  # get incoming stage for selected desk
            desk = app.data.find_one("desks", None, _id=desk_id)
            stage_id = desk["incoming_stage"] if desk else stage_id

        updates["task"] = {"desk": desk_id, "stage": stage_id, "user": None}

        updates[EXPIRY] = get_expiry(desk_id=desk_id, stage_id=stage_id)

        if doc[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE and doc.get(DELETED_GROUPS, None):
            updates[GROUPS] = doc[DELETED_GROUPS]
            updates[DELETED_GROUPS] = []

    def on_updated(self, updates, original):
        if original[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE and updates.get(GROUPS, None):
            # restore the deleted items from package
            package_service = PackageService()
            items = package_service.get_item_refs(updates)
            for item in items:
                package_item = get_resource_service(ARCHIVE).find_one(req=None, _id=item[GUID_FIELD])
                if package_item:
                    linked_in_packages = [
                        linked
                        for linked in package_item.get(LINKED_IN_PACKAGES, [])
                        if linked.get(PACKAGE) != original.get(config.ID_FIELD)
                    ]
                    linked_in_packages.append({PACKAGE: original.get(config.ID_FIELD)})
                    super().system_update(
                        package_item[config.ID_FIELD], {LINKED_IN_PACKAGES: linked_in_packages}, package_item
                    )

    def on_update(self, updates, original):
        updates[ITEM_OPERATION] = ITEM_UNSPIKE
        updates[ITEM_STATE] = original.get(REVERT_STATE)
        set_sign_off(updates, original=original)

    def update(self, id, updates, original):
        original_state = original[ITEM_STATE]
        if not is_workflow_state_transition_valid(ITEM_UNSPIKE, original_state):
            raise InvalidStateTransitionError()

        user = get_user(required=True)
        item = get_resource_service(ARCHIVE).find_one(req=None, _id=id)

        self.set_unspike_updates(item, updates)
        self.backend.update(self.datasource, id, updates, original)

        item = get_resource_service(ARCHIVE).find_one(req=None, _id=id)
        push_notification("item:unspike", item=str(id), user=str(user.get(config.ID_FIELD)))
        app.on_archive_item_updated(updates, original, ITEM_UNSPIKE)

        return item


superdesk.workflow_state("spiked")

superdesk.workflow_action(
    name="spike", exclude_states=["spiked", "published", "scheduled", "corrected", "killed"], privileges=["spike"]
)

superdesk.workflow_action(name="unspike", include_states=["spiked"], privileges=["unspike"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import superdesk

from superdesk.publish.formatters import Formatter
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, FORMAT, FORMATS
from flask import render_template
from copy import deepcopy
from superdesk.errors import FormatterError
from superdesk import etree as sd_etree


class EmailFormatter(Formatter):
    """Superdesk Email formatter.

    Feature media renditions are passed on to the transmit service, which if configured will attach the media to
    the email.

    It uses templates to render items, those can be overriden to customize the output:

    - ``email_article_subject.txt``
        email subject

    - ``email_article_body.txt``
        email text content

    - ``email_article_body.html``
        email html content

    It gets ``article`` with item data, can be used in templates like::

       <strong>{{ article.headline }}</strong>

    """

    def _inject_dateline(self, formatted_article):
        """Inject dateline in article's body_html"""
        body_html_elem = sd_etree.parse_html(formatted_article.get("body_html", "<p> </p>"))
        ptag = body_html_elem.find(".//p")
        if ptag is not None:
            ptag.text = formatted_article["dateline"]["text"] + " " + (ptag.text or "")
            formatted_article["body_html"] = sd_etree.to_string(body_html_elem)

    def format(self, article, subscriber, codes=None):
        formatted_article = deepcopy(article)
        pub_seq_num = superdesk.get_resource_service("subscribers").generate_sequence_number(subscriber)
        doc = {}
        try:
            if formatted_article.get(FORMAT) == FORMATS.HTML:
                if formatted_article.get("dateline", {}).get("text"):
                    # If there is a dateline inject it into the body
                    self._inject_dateline(formatted_article)
                doc["message_html"] = render_template("email_article_body.html", article=formatted_article)
            else:
                doc["message_html"] = None
            doc["message_text"] = render_template("email_article_body.txt", article=formatted_article)
            doc["message_subject"] = render_template("email_article_subject.txt", article=formatted_article)
            doc["renditions"] = ((formatted_article.get("associations", {}) or {}).get("featuremedia", {}) or {}).get(
                "renditions"
            )
        except Exception as ex:
            raise FormatterError.EmailFormatterError(ex, FormatterError)
        return [(pub_seq_num, json.dumps(doc))]

    def can_format(self, format_type, article):
        return format_type.lower() == "email" and article[ITEM_TYPE] == CONTENT_TYPE.TEXT



import logging

from contextlib import contextmanager
from superdesk.utils import Timer

logger = logging.getLogger(__name__)


@contextmanager
def timer(name):
    """Time logging context manager.

    Usage:

        with timer('name'):
            time.sleep(5)  # something you want to measure
    """
    _timer = Timer()
    _timer.start(name)
    yield _timer
    time = _timer.stop(name)
    logger.info("%s: %.3fms", name, time * 1000)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from bson import ObjectId
from unittest import mock
from unittest.mock import MagicMock

from celery.exceptions import SoftTimeLimitExceeded

import superdesk
from superdesk.tests import TestCase


class SoftTimeLimitTestCase(TestCase):
    @mock.patch("superdesk.publish.registered_transmitters")
    def test_softtimeout_exception(self, *mocks):
        self.app.config["MAX_TRANSMIT_RETRY_ATTEMPT"] = 4
        self.app.config["CELERY_TASK_ALWAYS_EAGER"] = False
        subscriber = {
            "_id": ObjectId("56c11bd78b84bb00b0a1905e"),
            "sequence_num_settings": {"max": 9999, "min": 1},
            "_etag": "f16b7eaa566f68b8d2561c811ec694bdf819784d",
            "is_active": True,
            "destinations": [{"delivery_type": "email"}],
            "email": "test@test.com",
            "subscriber_type": "digital",
            "name": "Test",
        }

        self.app.data.insert("subscribers", [subscriber])

        items = [
            {
                "_id": ObjectId(),
                "state": "pending",
                "item_id": "item_1",
                "item_version": 4,
                "headline": "pending headline",
                "destination": {"delivery_type": "email"},
                "subscriber_id": subscriber["_id"],
                "formatted_item": "test",
            },
            {
                "_id": ObjectId(),
                "state": "pending",
                "item_id": "item_2",
                "item_version": 4,
                "headline": "pending headline 2",
                "destination": {"delivery_type": "email"},
                "subscriber_id": subscriber["_id"],
                "formatted_item": "test",
            },
        ]

        self.app.data.insert("publish_queue", items)

        fake_transmitter = MagicMock()
        fake_transmitter.transmit.side_effect = SoftTimeLimitExceeded()

        fake_transmitters_list = mocks[0]
        fake_transmitters_list.__getitem__.return_value = fake_transmitter

        with self.assertRaises(SoftTimeLimitExceeded):
            superdesk.publish.publish_content.transmit_subscriber_items(subscriber.get("_id"), False)
        failed_item = self.app.data.find_one("publish_queue", req=None, _id=items[0].get("_id"))
        self.assertEqual(failed_item["state"], "retrying")
        pending_item = self.app.data.find_one("publish_queue", req=None, _id=items[1].get("_id"))
        self.assertEqual(pending_item["state"], "pending")
        self.app.config["CELERY_TASK_ALWAYS_EAGER"] = True



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from enum import Enum
import operator


class FilterConditionOperatorsEnum(Enum):
    in_ = (1,)
    nin = (2,)
    like = (3,)
    notlike = (4,)
    startswith = (5,)
    endswith = (6,)
    match = (7,)
    eq = (8,)
    ne = (9,)
    gt = (10,)
    gte = (11,)
    lt = (12,)
    lte = (13,)
    exists = 14


class FilterConditionOperator:
    @staticmethod
    def factory(operator):
        if operator + "_" == FilterConditionOperatorsEnum.in_.name:
            return InOperator(operator)
        elif operator == FilterConditionOperatorsEnum.nin.name:
            return NotInOperator(operator)
        elif operator == FilterConditionOperatorsEnum.notlike.name:
            return NotLikeOperator(operator)
        elif operator == FilterConditionOperatorsEnum.match.name:
            return MatchOperator(operator)
        elif operator in [
            FilterConditionOperatorsEnum.eq.name,
            FilterConditionOperatorsEnum.ne.name,
            FilterConditionOperatorsEnum.gt.name,
            FilterConditionOperatorsEnum.gte.name,
            FilterConditionOperatorsEnum.lt.name,
            FilterConditionOperatorsEnum.lte.name,
        ]:
            return ComparisonOperator(operator)
        elif operator == FilterConditionOperatorsEnum.exists.name:
            return ExistsOperator(operator)
        else:
            return RegexOperator(operator)

    def _get_default_mongo_operator(self):
        return "${}".format(self.operator.name)

    def get_mongo_operator(self):
        return self.mongo_operator

    def get_elastic_operator(self):
        return self.elastic_operator

    def contains_not(self):
        return False

    def does_match(self, article_value, filter_value):
        raise NotImplementedError()

    def get_lower_case(self, value):
        return str(value).lower()


class InOperator(FilterConditionOperator):
    def __init__(self, operator):
        self.operator = FilterConditionOperatorsEnum[operator + "_"]
        self.mongo_operator = "$in"
        self.elastic_operator = "terms"

    def does_match(self, article_value, filter_value):
        if isinstance(article_value, list):
            return any([self.get_lower_case(v) in map(self.get_lower_case, filter_value) for v in article_value])
        else:
            return self.get_lower_case(article_value) in map(self.get_lower_case, filter_value)


class NotInOperator(FilterConditionOperator):
    def __init__(self, operator):
        self.operator = FilterConditionOperatorsEnum[operator]
        self.mongo_operator = self._get_default_mongo_operator()
        self.elastic_operator = "terms"

    def does_match(self, article_value, filter_value):
        if isinstance(article_value, list):
            return all([self.get_lower_case(v) not in map(self.get_lower_case, filter_value) for v in article_value])
        else:
            return self.get_lower_case(article_value) not in map(self.get_lower_case, filter_value)

    def contains_not(self):
        return True


class NotLikeOperator(FilterConditionOperator):
    def __init__(self, operator):
        self.operator = FilterConditionOperatorsEnum[operator]
        self.mongo_operator = "$not"
        self.elastic_operator = "query_string"

    def does_match(self, article_value, filter_value):
        return filter_value.match(article_value) is None

    def contains_not(self):
        return True


class ComparisonOperator(FilterConditionOperator):
    """
    Represents comparison operators
    """

    _operators = {
        FilterConditionOperatorsEnum.gt: operator.gt,
        FilterConditionOperatorsEnum.lt: operator.lt,
        FilterConditionOperatorsEnum.gte: operator.ge,
        FilterConditionOperatorsEnum.lte: operator.le,
        FilterConditionOperatorsEnum.ne: operator.ne,
        FilterConditionOperatorsEnum.eq: operator.eq,
    }

    _elastic_mapper = {
        FilterConditionOperatorsEnum.gt: '{{"range": {{"{}": {{"gt": "{}"}}}}}}',
        FilterConditionOperatorsEnum.gte: '{{"range": {{"{}": {{"gte": "{}"}}}}}}',
        FilterConditionOperatorsEnum.lt: '{{"range": {{"{}": {{"lt": "{}"}}}}}}',
        FilterConditionOperatorsEnum.lte: '{{"range": {{"{}": {{"lte": "{}"}}}}}}',
        FilterConditionOperatorsEnum.eq: '{{"term": {{"{}": "{}"}}}}',
        FilterConditionOperatorsEnum.ne: '{{"term": {{"{}": "{}"}}}}',
    }

    def __init__(self, operator):
        self.operator = FilterConditionOperatorsEnum[operator]
        self.operator_func = ComparisonOperator._operators[FilterConditionOperatorsEnum[operator]]
        self.mongo_operator = self._get_default_mongo_operator()
        self.elastic_operator = ComparisonOperator._elastic_mapper[FilterConditionOperatorsEnum[operator]]

    def contains_not(self):
        return self.operator == FilterConditionOperatorsEnum.ne

    def does_match(self, article_value, filter_value):
        try:
            if isinstance(filter_value, bool):
                article_value = article_value.lower() in ("yes", "true", "t", "1")
            else:
                t = type(filter_value)
                article_value = t(article_value)

            if isinstance(filter_value, str):
                article_value = self.get_lower_case(article_value).strip()
                filter_value = self.get_lower_case(filter_value).strip()
            return self.operator_func(article_value, filter_value)
        except Exception:
            return False


class RegexOperator(FilterConditionOperator):
    """
    Represents In, StartsWith and EndsWith operators
    """

    def __init__(self, operator):
        self.operator = FilterConditionOperatorsEnum[operator]
        self.mongo_operator = "$regex"
        self.elastic_operator = "query_string"

    def does_match(self, article_value, filter_value):
        return filter_value.match(article_value) is not None


class MatchOperator(FilterConditionOperator):
    def __init__(self, operator):
        self.operator = FilterConditionOperatorsEnum[operator]
        self.mongo_operator = "$in"
        self.elastic_operator = '{{"query_string": {{"{}":"{}"}}}}'

    def does_match(self, article_value, filter_value):
        if isinstance(article_value, list):
            return any([self.get_lower_case(v) in map(self.get_lower_case, filter_value) for v in article_value])
        else:
            return self.get_lower_case(article_value) in map(self.get_lower_case, filter_value)


class ExistsOperator(FilterConditionOperator):
    def __init__(self, operator):
        self.operator = FilterConditionOperatorsEnum[operator]
        self.get_mongo_operator = "$exists"
        self.get_elastic_operator = '{{"exists": {{"field": "{}"}} }}'

    def does_match(self, article_value, filter_value):
        # exists
        if filter_value:
            return article_value
        # does no exist
        else:
            return not article_value



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .length_yards_to_metric import yards_to_metric


class YardTestCase(unittest.TestCase):
    def test_y_to_m(self):
        text = (
            "Tom runs 125 yd in 11 seconds. "
            "Tom runs 125 yards in 11 seconds. "
            "Tom runs 125 Yards in 11 seconds. "
            "Tom runs 125-Yards in 11 seconds. "
            "Tom runs 12.5 yd in 11 seconds. "
            "Tom runs 12,500 yards in 11 seconds. "
            "Tom runs 100-12500 Yards in 11 seconds. "
        )
        item = {"body_html": text}
        res, diff = yards_to_metric(item)
        self.assertEqual(diff["125 yd"], "125 yd (114 metres)")
        self.assertEqual(diff["125 yards"], "125 yards (114 metres)")
        self.assertEqual(diff["125 Yards"], "125 Yards (114 metres)")
        self.assertEqual(diff["125-Yards"], "125-Yards (114 metres)")
        self.assertEqual(diff["12.5 yd"], "12.5 yd (11.4 metres)")
        self.assertEqual(diff["12,500 yards"], "12,500 yards (11,430 metres)")
        self.assertEqual(diff["100-12500 Yards"], "100-12500 Yards (91-11,430 metres)")
        self.assertEqual(res["body_html"], item["body_html"])



from superdesk.resource import Resource
from superdesk.metadata.utils import item_url
from superdesk.auth_server.scopes import Scope


class UsersResource(Resource):
    url = "users"
    item_url = item_url
    item_methods = ["GET"]
    resource_methods = ["GET"]
    allow_unknown = True
    datasource = {"source": "users", "default_sort": [("username", 1)]}
    privileges = {"GET": Scope.USERS_READ.name}



import os
import superdesk

from flask import current_app
from werkzeug.utils import secure_filename
from apps.auth import get_user_id


RESOURCE = "attachments"


class AttachmentsResource(superdesk.Resource):
    schema = {
        "media": {"type": "media"},
        "mimetype": {"type": "string"},
        "filename": {"type": "string"},
        "length": {"type": "integer"},
        "title": {"type": "string"},
        "description": {"type": "string"},
        "user": superdesk.Resource.rel("users"),
        "internal": {"type": "boolean", "default": False},
    }

    item_methods = ["GET", "PATCH"]
    resource_methods = ["GET", "POST"]
    privileges = {"POST": "archive", "PATCH": "archive"}


class AttachmentsService(superdesk.Service):
    def on_create(self, docs):
        for doc in docs:
            doc["user"] = get_user_id()
            if doc.get("media"):
                media = current_app.media.get(doc["media"], RESOURCE)
                doc.setdefault("filename", secure_filename(os.path.basename(getattr(media, "filename"))))
                doc.setdefault("mimetype", getattr(media, "content_type"))
                doc.setdefault("length", getattr(media, "length"))

    def on_deleted(self, doc):
        current_app.media.delete(doc["media"], RESOURCE)


def is_attachment_public(attachment):
    """Retuns true if attachment is public. False if it's internal.

    :param attachment: Attachment object or id inside attachment attribute
    :return: boolean
    """
    if attachment.get("attachment"):  # retrieve object reference
        attachment = superdesk.get_resource_service("attachments").find_one(req=None, _id=attachment["attachment"])

    return not attachment.get("internal")


def init_app(app):
    superdesk.register_resource(RESOURCE, AttachmentsResource, AttachmentsService)
    app.client_config["attachments_max_files"] = app.config.get("ATTACHMENTS_MAX_FILES", 10)
    app.client_config["attachments_max_size"] = app.config.get("ATTACHMENTS_MAX_SIZE", 2 ** 20 * 8)  # 8MB



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from decimal import Decimal
from . import unit_base
from flask_babel import lazy_gettext

CUBIC_METER_SYMBOL = "cubic meter"


def convert(cubic_yard, precision=1):
    """
    Converts from cubic yard to cubic m
    :param cubic_yard: Cubic yard value in string
    :return: cubic m value in string, and the symbol
    """
    cy_to_cm_rate = Decimal(0.764555)
    symbol = CUBIC_METER_SYMBOL
    cubic_yard_list = cubic_yard.split("-")
    cubic_meter_list = [unit_base.format_converted(Decimal(a) * cy_to_cm_rate, precision) for a in cubic_yard_list]
    return "-".join(cubic_meter_list), symbol


def cubic_yard_to_metric(item, **kwargs):
    """Converts cubic yard values to metric"""

    regex = r"(\d+-?,?\.?\d*)((\s*)|(-))((cu\.?\s*-?yds?)|(cb\.?\s*-?yds?)|([cC]ubic\s*-?([yY]ards?|yds?)))\b"
    return unit_base.do_conversion(item, convert, unit_base.format_output, regex, match_index=0, value_index=1)


name = "cubic_yard_to_metric"
label = lazy_gettext("Volume cubic yard to metric")
callback = cubic_yard_to_metric
access_type = "frontend"
action_type = "interactive"
group = lazy_gettext("volume")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import current_app
from .commands import RemoveExpiredItems  # noqa


MONGO_PREFIX = "CONTENTAPI_MONGO"
ELASTIC_PREFIX = "CONTENTAPI_ELASTICSEARCH"


def is_enabled():
    """Test if content api is enabled.

    It can be turned off via ``CONTENTAPI_ENABLED`` setting.
    """
    return current_app.config.get("CONTENTAPI_ENABLED", True)



import superdesk.schema as schema


class DefaultSchema(schema.Schema):
    """Default schema."""

    #: usageterms
    usageterms = schema.StringField()

    #: keywords
    keywords = schema.ListField()

    #: language
    language = schema.StringField()

    #: slugline
    slugline = schema.StringField(maxlength=24)

    #: item genre
    genre = schema.ListField()

    #: anpa take key
    anpa_take_key = schema.StringField()

    #: place where news happened
    place = schema.ListField()

    #: news item priority
    priority = schema.IntegerField()

    #: news item urgency
    urgency = schema.IntegerField()

    #: category
    anpa_category = schema.ListField()

    #: subject
    subject = schema.ListField(
        required=True,
        mandatory_in_list={"scheme": {}},
        schema={
            "type": "dict",
            "schema": {
                "name": {},
                "qcode": {},
                "scheme": {"type": "string", "required": True, "nullable": True, "allowed": []},
                "service": {"nullable": True},
                "parent": {"nullable": True},
            },
        },
    )

    #: company codes
    company_codes = schema.ListField()

    #: editorial note
    ednote = schema.StringField()

    #: authors
    authors = schema.ListField(
        schema={
            "type": "dict",
            "schema": {
                "name": {"type": "string"},
                "parent": {"type": "string"},
                "role": {"type": "string"},
            },
        }
    )

    #: headline
    headline = schema.StringField(maxlength=64)

    #: sms version of an item
    sms = schema.StringField()

    #: item abstract
    abstract = schema.StringField(maxlength=160)

    #: byline
    byline = schema.StringField()

    #: dateline - info about where news was written
    dateline = schema.DictField()

    #: item content
    body_html = schema.StringField()

    #: item footer
    footer = schema.StringField()

    #: body footer
    body_footer = schema.StringField()

    #: item sign off info
    sign_off = schema.StringField()

    #: embedded media in the item
    feature_media = schema.MediaField()

    #: embedded media description
    media_description = schema.StringField()

    #: item attachments
    #: .. versionadded:: 1.29
    attachments = schema.ListField()


DEFAULT_SCHEMA = dict(DefaultSchema)


DEFAULT_EDITOR = {
    "slugline": {"order": 1, "sdWidth": "full", "enabled": True},
    "keywords": {"order": 2, "sdWidth": "full", "enabled": False},
    "language": {"order": 3, "sdWidth": "half", "enabled": False},
    "usageterms": {"order": 4, "sdWidth": "full", "enabled": False},
    "genre": {"order": 5, "sdWidth": "half", "enabled": True},
    "anpa_take_key": {"order": 6, "sdWidth": "half", "enabled": False},
    "place": {"order": 7, "sdWidth": "half", "enabled": True},
    "priority": {"order": 8, "sdWidth": "quarter", "enabled": True},
    "urgency": {"order": 9, "sdWidth": "quarter", "enabled": True},
    "anpa_category": {"order": 10, "sdWidth": "full", "enabled": True},
    "subject": {"order": 11, "sdWidth": "full", "enabled": True},
    "company_codes": {"order": 12, "sdWidth": "full", "enabled": False},
    "ednote": {"order": 13, "sdWidth": "full", "enabled": True},
    "authors": {"order": 14, "sdWidth": "full", "enabled": True},
    "headline": {"order": 15, "formatOptions": [], "enabled": True},
    "sms": {"order": 16, "enabled": False},
    "abstract": {
        "order": 17,
        "formatOptions": ["bold", "italic", "underline", "link"],
        "editor3": True,
        "enabled": True,
    },
    "byline": {"order": 18, "enabled": True},
    "dateline": {"order": 19, "enabled": True},
    "body_html": {
        "order": 20,
        "formatOptions": ["h2", "bold", "italic", "underline", "quote", "link", "embed", "media"],
        "cleanPastedHTML": False,
        "editor3": True,
        "enabled": True,
    },
    "footer": {"order": 21, "enabled": False},
    "body_footer": {"order": 22, "enabled": False},
    "sign_off": {"order": 23, "enabled": True},
    "feature_media": {"enabled": True},
    "media_description": {"enabled": True},
    "attachments": {"enabled": False},
}



import io
import zipfile
from lxml import etree

from . import Mimetype, Preferences, Styles, Tags, Story, StoryTable, StoryList, Spread, Designmap, Graphic


class Converter:
    """
    IDML Converter.

    Format items to `IDML <https://fileinfo.com/extension/idml>`
    """

    ZIP_COMPRESSION = zipfile.ZIP_STORED
    PAGE_MARGIN_TOP = 50
    PAGE_MARGIN_BOTTOM = 50
    PAGE_MARGIN_LEFT = 50
    PAGE_MARGIN_RIGHT = 50
    DOCUMENT_PAGE_HEIGHT = 841.8897637776
    DOCUMENT_PAGE_WIDTH = 595.2755905488
    DOCUMENT_PAGE_INNER_WIDTH = DOCUMENT_PAGE_WIDTH - PAGE_MARGIN_LEFT - PAGE_MARGIN_RIGHT

    def __init__(self):
        self._idml_bytes_buffer = None
        self._in_memory_zip = None
        self._package = []
        self._counter = {}

    def create_idml(self, article):
        """
        Creates idml file as in-memory zip container.

        IDML file is a zip container and it inludes the next files and folders:
         - mimetype
         - designmap.xml:
            Is the key to all of the other filles that appear within the IDML package.
         - XML:
            Folder contains XML elements and settings used in the InDesign document.
         - XML/Tags.xml:
            Contains the XML tag defnitions stored in the InDesign document.
         - Resources:
            Folder contains elements that are commonly used by other filles within the document,
            such as colors, fonts, and paragraph styles.
         - Resources/Graphic.xml:
            Contains the inks, colors, swatches, gradients, mixed inks, mixed ink groups, tints,
            and stroke styles contained in the document.
         - Resources/Preferences.xml:
            Contains representations of all of the document preferences.
         - Resources/Styles.xml:
            Contains all of the paragraph, character, object, cell, table, and TOC styles used in the document.
         - Spreads:
            Folder contains the XML  les representing the spreads in the document.
         - Spreads/Spread_spread_*.xml:
            Each spread contains all of the page items (rectangles, ellipses, graphic lines, polygons, groups,
            buttons, and text frames) that appear on the pages of the spread.
         - Stories:
            Folder contains all of the stories in the InDesign document.
         - Stories/Story_story_*.xml:
            Represents the contents of a single story and all of the formatting attributes applied to the
            text in the story.

        :param dict article: article data
        :return bytes: idml file as a bytes
        """
        self._counter = {"spread": 0, "page": 0, "story": 0, "textframe": 0}
        self._init_zip_container()
        self._package = []
        self._package.append(Mimetype())
        self._package.append(Styles())
        self._package.append(Graphic())
        self._package.append(
            Preferences(
                attributes={
                    "DocumentPreference": {
                        "PageHeight": str(self.DOCUMENT_PAGE_HEIGHT),
                        "PageWidth": str(self.DOCUMENT_PAGE_WIDTH),
                        "PagesPerDocument": "1",
                        "FacingPages": "false",
                    }
                }
            )
        )
        self._package.append(
            Tags(
                attributes={
                    "Headline": {"TagColor": "Orange"},
                    "Byline": {"TagColor": "Orange"},
                    "Heading1": {"TagColor": "Red"},
                    "Heading2": {"TagColor": "Red"},
                    "Heading3": {"TagColor": "Red"},
                    "Heading4": {"TagColor": "Red"},
                    "Heading5": {"TagColor": "Red"},
                    "Heading6": {"TagColor": "Red"},
                    "NormalParagraph": {"TagColor": "Green"},
                    "Blockquote": {"TagColor": "Blue"},
                    "Preformatted": {"TagColor": "Black"},
                    "Table": {"TagColor": "Yellow"},
                    "UnorderedList": {"TagColor": "Pink"},
                    "OrderedList": {"TagColor": "Pink"},
                }
            )
        )
        self._create_stories(article)
        self._create_spreads()
        self._create_designmap()
        self._write_package()
        self._in_memory_zip.close()

        return self._idml_bytes_buffer.getvalue()

    def _init_zip_container(self):
        """
        Creates zipfile with `x` (exclusively create and write a new file) mode.
        BytesIO file-like object is used for zipfile file reference.
        """
        self._idml_bytes_buffer = io.BytesIO()
        self._in_memory_zip = zipfile.ZipFile(self._idml_bytes_buffer, mode="x", compression=self.ZIP_COMPRESSION)

    def _write_package(self):
        """
        Write files into zip container.

        ``self._package`` includes only objects based on
        ``<class 'superdesk.publish.formatters.idml_formatter.package.base.BasePackageElement'>``,
        they know how to render itself (file content) and how to name itself.
        """
        for item in self._package:
            self._in_memory_zip.writestr(item.filename, item.render())

    def _create_stories(self, article):
        """
        Walk thrrough `article['body_html']` and creates a Story/StoryTable/StoryList instances for each element.
        :param dict article: article data
        """

        body_html = article["body_html"]

        if article.get("byline"):
            byline = etree.Element("byline")
            byline.text = article.get("byline")
            body_html = etree.tostring(byline, pretty_print=False).decode("utf-8") + body_html
        if article.get("headline"):
            headline = etree.Element("headline")
            headline.text = article.get("headline")
            body_html = etree.tostring(headline, pretty_print=False).decode("utf-8") + body_html

        parser = etree.HTMLParser(recover=True, remove_blank_text=True)
        root = etree.fromstring(body_html, parser)
        body = root.find("body")

        for element in body:
            if element.tag in ("p", "h1", "h2", "h3", "h4", "h5", "h6", "blockquote", "pre", "headline", "byline"):
                # create text story
                self._package.append(
                    Story(
                        self._next_story_id(),
                        element,
                        attributes={
                            "ParagraphStyleRange": Story.BLOCK_TAGS_MAPPING[element.tag]["ParagraphStyleRange"]
                        },
                        markup_tag=Story.BLOCK_TAGS_MAPPING[element.tag]["markup_tag"],
                    )
                )
            elif element.tag in ("ul", "ol"):
                # create list story
                self._package.append(
                    StoryList(
                        self._next_story_id(),
                        element,
                        attributes={
                            "ParagraphStyleRange": Story.BLOCK_TAGS_MAPPING[element.tag]["ParagraphStyleRange"]
                        },
                        markup_tag=Story.BLOCK_TAGS_MAPPING[element.tag]["markup_tag"],
                    )
                )
            elif element.tag == "table":
                # create table story
                self._package.append(
                    StoryTable(
                        self._next_story_id(),
                        element,
                        self.DOCUMENT_PAGE_INNER_WIDTH,
                        attributes={"Table": Story.BLOCK_TAGS_MAPPING[element.tag]["Table"]},
                        markup_tag=Story.BLOCK_TAGS_MAPPING[element.tag]["markup_tag"],
                    )
                )

    def _create_spreads(self):
        """
        Walk through all stories, create a spread(s) and place stories into strpead(s).
        """
        active_spread = self._create_spread_with_page()

        # walk through packages and place them into spread->page
        for package in self._package:
            _type = type(package)
            if _type in (Story, StoryTable, StoryList):
                # guess height for a text frame (yes, table and lists are also textframes)
                text_frame_height = _type.guess_height(package, self.DOCUMENT_PAGE_INNER_WIDTH)

                # if guessed height for text frame is more than page height
                if text_frame_height > active_spread.page_inner_height:
                    text_frame_height = active_spread.page_inner_height

                # check if textframe will fit current page
                if not active_spread.check_if_fits(text_frame_height):
                    active_spread = self._create_spread_with_page()

                # place text frame
                active_spread.place_textframe(
                    height=text_frame_height,
                    attributes={
                        "TextFrame": {
                            "Self": self._next_textframe_id(),
                            "ParentStory": package.self_id,
                        }
                    },
                )

    def _create_designmap(self):
        """
        Create `Designmap` instance add it to the package.
        """
        designmap = Designmap()

        # let's fill designmap
        # order plays a big role
        for _type in (Graphic, Styles, Preferences, Tags, Spread, Story, StoryTable, StoryList):
            for obj in [i for i in self._package if i.__class__ is _type]:
                designmap.add_pkg(_type.__name__, obj.filename)

        for _type in (Story, StoryTable, StoryList):
            for obj in [i for i in self._package if i.__class__ is _type]:
                designmap.add_hyperlinks(obj.links)

        self._package.append(designmap)

    def _create_spread_with_page(self):
        """
        Create spread and put one page inside.
        :return Spread: spread instance
        """
        spread = Spread(
            self._next_spread_id(),
            document_page_width=self.DOCUMENT_PAGE_WIDTH,
            document_page_height=self.DOCUMENT_PAGE_HEIGHT,
        )
        self._package.append(spread)

        # create page
        page_id = self._next_page_id()
        spread.add_page(
            {
                "Page": {"Self": page_id, "Name": page_id, "UseMasterGrid": "false"},
                "MarginPreference": {
                    "Top": str(self.PAGE_MARGIN_TOP),
                    "Bottom": str(self.PAGE_MARGIN_BOTTOM),
                    "Left": str(self.PAGE_MARGIN_LEFT),
                    "Right": str(self.PAGE_MARGIN_RIGHT),
                },
            }
        )

        return spread

    def _next_story_id(self):
        """
        Generate a story id.
        :return str: story id
        """
        self_id = "story_{}".format(self._counter["story"])
        self._counter["story"] += 1
        return self_id

    def _next_spread_id(self):
        """
        Generate a spread id.
        :return str: spread id
        """
        self_id = "spread_{}".format(self._counter["spread"])
        self._counter["spread"] += 1
        return self_id

    def _next_page_id(self):
        """
        Generate a page id.
        :return str: page id
        """
        self_id = "page_{}".format(self._counter["page"])
        self._counter["page"] += 1
        return self_id

    def _next_textframe_id(self):
        """
        Generate a textframe id.
        :return str: textframe id
        """
        self_id = "textframe_{}".format(self._counter["textframe"])
        self._counter["textframe"] += 1
        return self_id



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .weight_pounds_to_metric import pounds_to_metric


class VolumeTestCase(unittest.TestCase):
    def test(self):
        text = (
            "Total volume is 100.50 pounds for this land. "
            "Total volume is 15.7 lbs. for this land. "
            "Total volume is 1 Pound for this land. "
            "Total volume is 1-16 lb for this land. "
            "Total volume is 1-16 lbs for this land. "
            "Total volume is 16.7-pound for this land. "
            "Total volume is 16,500-lb. for this land. "
        )
        item = {"body_html": text}
        res, diff = pounds_to_metric(item)
        self.assertEqual(diff["100.50 pounds"], "100.50 pounds (45.59 kg)")
        self.assertEqual(diff["15.7 lbs"], "15.7 lbs (7.1 kg)")
        self.assertEqual(diff["1 Pound"], "1 Pound (0.45 kg)")
        self.assertEqual(diff["1-16 lb"], "1-16 lb (0.45-7 kg)")
        self.assertEqual(diff["1-16 lbs"], "1-16 lbs (0.45-7 kg)")
        self.assertEqual(diff["16.7-pound"], "16.7-pound (7.6 kg)")
        self.assertEqual(diff["16,500-lb"], "16,500-lb (7,484 kg)")
        self.assertEqual(res["body_html"], item["body_html"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Superdesk IO Registry"""

from superdesk import Resource, Service
from superdesk.utils import ListCursor
from superdesk.errors import SuperdeskIngestError, AlreadyExistsError

registered_feed_parsers = {}
allowed_feed_parsers = []

registered_feeding_services = {}
allowed_feeding_services = []
feeding_service_errors = {}
publish_errors = []
restricted_feeding_service_parsers = {}


def register_feeding_service(service_class):
    """
    Registers the Feeding Service with the application.

    :class: `superdesk.io.feeding_services.RegisterFeedingService` uses this function to register the feeding service.

    :param service_class: Feeding Service class
    :raises: AlreadyExistsError if a feeding service with same name already been registered
    """

    if service_class.NAME in registered_feeding_services:
        raise AlreadyExistsError(
            "Feeding Service: {} already registered by {}".format(
                service_class.NAME, registered_feeding_services[service_class.NAME]
            )
        )

    registered_feeding_services[service_class.NAME] = service_class
    allowed_feeding_services.append(service_class.NAME)

    service_class.ERRORS.append(SuperdeskIngestError.parserNotFoundError().get_error_description())
    feeding_service_errors[service_class.NAME] = dict(service_class.ERRORS)


def register_feeding_service_error(service_name, error):
    """
    Registers an error with the service named

    :param service_name: unique name to identify the Feeding Service class
    :param error: tuple representing the error, the tuple contains the error_code and the error message
    :return:
    """
    feeding_service_errors.get(service_name, {}).update(dict([error]))


def get_feeding_service(service_name):
    """
    Create and return Feeding Service instance.
    :param service_name: unique name to identify the Feeding Service class.
    :return: Feeding Service instance.
    :raise KeyError: there is no feeding service registered with this `service_name`.
    """

    return registered_feeding_services[service_name]()


def register_feed_parser(parser_name, parser_class):
    """
    Registers the Feed Parser with the application.

    :class: `superdesk.io.feed_parsers.RegisterFeedParser` uses this function to register the feed parser.

    :param parser_name: unique name to identify the Feed Parser class
    :param parser_class: Feed Parser class
    :raises: AlreadyExistsError if a feed parser with same name already been registered
    """

    if parser_name in registered_feed_parsers:
        raise AlreadyExistsError(
            "Feed Parser: {} already registered by {}".format(parser_name, type(registered_feed_parsers[parser_name]))
        )

    registered_feed_parsers[parser_name] = parser_class
    allowed_feed_parsers.append(parser_name)


def register_feeding_service_parser(service_name, parser_name):
    """
    Registers the Feed Parser with the Feeding service.

    :param service_name: unique name to identify the Feeding Service class
    :param parser_name: unique name to identify the Feed Parser class
        None if the feeding service doesn't expect any parser
    """
    if not restricted_feeding_service_parsers.get(service_name):
        restricted_feeding_service_parsers[service_name] = {}
    if parser_name is None:
        if restricted_feeding_service_parsers[service_name]:
            raise ValueError("You can't set None to a feeding service if some parsers are already registered")
        restricted_feeding_service_parsers[service_name] = None
    else:
        restricted_feeding_service_parsers[service_name][parser_name] = True


def get_feed_parser(parser_name):
    """
    Retrieve registered feed parser class from its name

    :param parser_name: name of the parser, as in register_feed_parser
    :type parser_name: str
    :return str: feed parser class
    :raise KeyError: there is no parser registered with this name
    """
    return registered_feed_parsers[parser_name]


class FeedParserAllowedResource(Resource):
    resource_methods = ["GET"]
    item_methods = []
    allow_unknown = True


class FeedParserAllowedService(Service):
    def get(self, req, lookup):
        def parser(parser_id):
            registered = registered_feed_parsers[parser_id]
            return {"feed_parser": parser_id, "label": getattr(registered, "label", parser_id)}

        return ListCursor([parser(_id) for _id in registered_feed_parsers])


class FeedingServiceAllowedResource(Resource):
    resource_methods = ["GET"]
    item_methods = []
    schema = {
        "feeding_service": {"type": "string"},
        "label": {"type": "string"},
        "fields": {"type": "list"},
        "field_groups": {"type": "dict", "schema": {}},
        "parser_restricted_values": {"type": "list"},
    }


class FeedingServiceAllowedService(Service):
    def get(self, req, lookup):
        def service(service_id):
            feeding_service_class = registered_feeding_services[service_id]
            restricted_parsers = restricted_feeding_service_parsers.get(service_id, {})
            if restricted_parsers is not None:
                restricted_parsers = list(restricted_parsers.keys())

            fields = getattr(feeding_service_class, "fields", [])

            return {
                "feeding_service": service_id,
                "label": getattr(feeding_service_class, "label", service_id),
                "fields": fields,
                "field_groups": getattr(feeding_service_class, "field_groups", {}),
                "parser_restricted_values": restricted_parsers,
            }

        return ListCursor([service(_id) for _id in registered_feeding_services])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.io.feed_parsers.image_iptc import ImageIPTCFeedParser
from superdesk.tests import TestCase
import os


class BaseImageIPTCTestCase(TestCase):
    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        self.image_path = os.path.normpath(os.path.join(dirname, "../fixtures", self.filename))
        provider = {"name": "Test"}
        parser = ImageIPTCFeedParser()
        self.item = parser.parse(self.image_path, provider)


class ImageIPTCTestCase(BaseImageIPTCTestCase):
    filename = "IPTC-PhotometadataRef-Std2017.1.jpg"

    def test_can_parse(self):
        self.assertTrue(ImageIPTCFeedParser().can_parse(self.image_path))

    def test_content(self):
        item = self.item
        self.assertNotIn("_id", item)
        self.assertEqual(item["headline"], "The Headline (ref2017.1)")
        self.assertEqual(item["byline"], "Creator1 (ref2017.1)")
        self.assertEqual(item["slugline"], "The Title (ref2017.1)")
        self.assertEqual(
            item["description_text"],
            "The description aka caption (ref2017.1)",
        )
        self.assertEqual(item["keywords"], ["Keyword1ref2017.1", "Keyword2ref2017.1", "Keyword3ref2017.1"])
        self.assertEqual(item["ednote"], "An Instruction (ref2017.1)")
        self.assertEqual(item["copyrightnotice"], "Copyright (Notice) 2017.1 IPTC - www.iptc.org  (ref2017.1)")
        self.assertEqual(item["assignment_id"], "Job Id (ref2017.1)")
        self.assertEqual(item["firstcreated"].isoformat(), "2017-07-13T17:01:00+00:00")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from lxml import etree
from superdesk.io.feed_parsers.scoop_newsml_2_0 import ScoopNewsMLTwoFeedParser
from superdesk.tests import TestCase


class ScoopTestCase(TestCase):
    filename = "scoop.xml"
    vocab = [
        {
            "_id": "locators",
            "items": [
                {
                    "is_active": True,
                    "name": "NZ",
                    "world_region": "Oceania",
                    "country": "New Zealand",
                    "state": "",
                    "qcode": "NZ",
                    "group": "Rest Of World",
                }
            ],
        }
    ]

    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", self.filename))
        provider = {"name": "Test"}
        with self.app.app_context():
            self.app.data.insert("vocabularies", self.vocab)
        with open(fixture, "rb") as f:
            parser = ScoopNewsMLTwoFeedParser()
            self.xml = etree.parse(f)
            self.item = parser.parse(self.xml.getroot(), provider)

    def test_content(self):
        self.assertEqual(
            self.item[0].get("headline"),
            "Dick Smith receivers install Don Grover as acting CEO as Abboud " + "quits; retailer put up for sale",
        )
        self.assertEqual(self.item[0].get("guid"), "BD-20160112123540:3")
        self.assertEqual(self.item[0].get("uri"), "BD-20160112123540")
        self.assertEqual(self.item[0].get("priority"), 6)
        self.assertEqual(self.item[0].get("version"), "3")
        self.assertEqual(self.item[0].get("firstcreated").isoformat(), "2016-01-11T23:35:40+00:00")
        self.assertEqual(self.item[0].get("urgency"), 3)
        self.assertTrue(self.item[0].get("body_html").startswith("<p>The receivers of Dick Smith Holdings"))
        self.assertEqual(self.item[0].get("byline"), "Jonathan Underhill")
        self.assertEqual(self.item[0].get("dateline").get("located").get("city"), "Wellington")
        self.assertEqual(self.item[0].get("place")[0].get("name"), "NZ")

    def test_can_parse(self):
        self.assertTrue(ScoopNewsMLTwoFeedParser().can_parse(self.xml.getroot()))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk
from superdesk.tests import TestCase
from apps.search_providers.service import SearchProviderService
from apps.search_providers import register_search_provider
from superdesk.errors import AlreadyExistsError


class SearchProviderTestCase(TestCase):
    def setUp(self):
        with self.app.app_context():
            self.service = SearchProviderService("search_providers", backend=superdesk.get_backend())
            try:
                register_search_provider("provider1", "provider1")
                register_search_provider("provider2", "provider2")
            except AlreadyExistsError:
                pass

    def test_set_is_default_on_create(self):
        with self.app.app_context():
            provider1 = {"search_provider": "provider1", "source": "provider1", "is_default": True}
            provider2 = {"search_provider": "provider2", "source": "provider2", "is_default": True}
            provider1_id = self.service.post([provider1])[0]
            provider2_id = self.service.post([provider2])[0]
            provider1 = self.service.find_one(req=None, _id=provider1_id)
            provider2 = self.service.find_one(req=None, _id=provider2_id)
            self.assertEqual(provider1["is_default"], False)
            self.assertEqual(provider2["is_default"], True)

    def test_set_is_default_on_update(self):
        with self.app.app_context():
            provider1 = {"search_provider": "provider1", "source": "provider1", "is_default": False}
            provider2 = {"search_provider": "provider2", "source": "provider2", "is_default": True}
            provider1_id = self.service.post([provider1])[0]
            provider2_id = self.service.post([provider2])[0]
            self.service.patch(provider1_id, {"is_default": True})
            provider1 = self.service.find_one(req=None, _id=provider1_id)
            provider2 = self.service.find_one(req=None, _id=provider2_id)
            self.assertEqual(provider1["is_default"], True)
            self.assertEqual(provider2["is_default"], False)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource

from .enums import ConceptNature


CONCEPT_ITEMS_PRIVELEGE = "concept_items"


class ConceptItemsResource(Resource):
    """Concept item schema"""

    schema = {
        "name": {"type": "string", "required": True, "empty": False},
        "group_id": {"type": "string", "required": False, "empty": False, "readonly": False},
        "definition_text": {"type": "string", "required": False, "empty": False, "readonly": True},
        "definition_html": {"type": "string", "required": True, "empty": False},
        "language": {"type": "string", "required": True, "empty": False},
        "labels": {"type": "list", "schema": {"type": "string"}, "unique_list": True},
        # http://cv.iptc.org/newscodes/cpnature/
        "cpnat_type": {"type": "string", "required": True, "allowed": ConceptNature.values()},
        # https://iptc.org/std/NewsML-G2/guidelines/#more-real-world-entities
        "properties": {"type": "dict", "required": False},
        "created_by": Resource.rel("users", embeddable=False, readonly=True),
        "updated_by": Resource.rel("users", embeddable=False, readonly=True),
        # system fields
        "_type": {"type": "string"},
    }
    privileges = {
        "POST": CONCEPT_ITEMS_PRIVELEGE,
        "PATCH": CONCEPT_ITEMS_PRIVELEGE,
        "PUT": CONCEPT_ITEMS_PRIVELEGE,
        "DELETE": CONCEPT_ITEMS_PRIVELEGE,
    }
    item_url = 'regex("[a-f0-9]{24}")'
    resource_methods = ["GET", "POST"]
    item_methods = ["GET", "PATCH", "PUT", "DELETE"]
    query_objectid_as_string = False
    mongo_indexes = {
        "name_collation": ([("name", 1)], {"collation": {"locale": "en", "strength": 1}}),
        "definition_text_collation": (
            [
                ("definition_text", 1),
            ],
            {"collation": {"locale": "en", "strength": 1}},
        ),
        "group_id": ([("group_id", 1), ("language", 1)], {"unique": True}),
    }



import os
import time

from datetime import timedelta
from unittest.mock import patch, Mock

from superdesk.utc import utcnow
from superdesk.tests import TestCase
from superdesk.storage import AmazonMediaStorage
from superdesk.media.media_operations import guess_media_extension


class AmazonMediaStorageTestCase(TestCase):
    def setUp(self):
        self.amazon = AmazonMediaStorage(self.app)

        # Patch config with defaults
        p = patch.dict(
            self.app.config,
            {
                "AMAZON_SECRET_ACCESS_KEY": None,
                "AMAZON_CONTAINER_NAME": "acname",
                "AMAZON_REGION": "us-east-1",
                "AMAZON_S3_SUBFOLDER": "",
                "MEDIA_PREFIX": "https://acname.s3-us-east-1.amazonaws.com",
            },
        )
        p.start()
        self.addCleanup(p.stop)

    def test_media_url(self):
        filename = "test"
        # automatic version is set on 15mins granularity.
        mins_granularity = int(int(time.strftime("%M")) / 4) * 4
        time_id = "%s%s" % (time.strftime("%Y%m%d%H%m"), mins_granularity)
        media_id = self.amazon.media_id(filename)
        self.assertEqual("%s/%s" % (time_id, filename), media_id)
        self.assertEqual(self.amazon.url_for_media(media_id), "https://acname.s3-us-east-1.amazonaws.com/%s" % media_id)
        sub = "test-sub"
        settings = {"AMAZON_S3_SUBFOLDER": sub, "MEDIA_PREFIX": "https://acname.s3-us-east-1.amazonaws.com/" + sub}
        with patch.dict(self.app.config, settings):
            media_id = self.amazon.media_id(filename)
            self.assertEqual("%s/%s" % (time_id, filename), media_id)
            path = "%s/%s" % (sub, media_id)
            self.assertEqual(self.amazon.url_for_media(media_id), "https://acname.s3-us-east-1.amazonaws.com/%s" % path)
            with patch.object(self.amazon, "client") as s3:
                self.amazon.get(media_id)
                self.assertTrue(s3.get_object.called)
                self.assertEqual(s3.get_object.call_args[1], dict(Bucket="acname", Key=path))

    def test_put_and_delete(self):
        """Test amazon if configured.

        If the environment variables have a Amazon secret key set then assume
        that we can attempt to put and delete into s3

        :return:
        """
        if self.app.config["AMAZON_SECRET_ACCESS_KEY"]:
            id = self.amazon.put("test", content_type="text/plain")
            self.assertIsNot(id, None)
            self.assertTrue(self.amazon.exists(id))
            fromS3 = self.amazon.get(id)
            self.assertEqual(fromS3.read().decode("utf-8"), "test")
            self.amazon.delete(id)
            self.assertFalse(self.amazon.exists(id))
        else:
            self.assertTrue(True)

    def test_put_into_folder(self):
        data = b"test data"
        folder = "s3test"
        filename = "abc123.zip"
        content_type = "application/zip"
        self.amazon.client.put_object = Mock()
        self.amazon.media_id = Mock(return_value=filename)
        self.amazon._check_exists = Mock(return_value=False)

        self.amazon.put(data, filename, content_type, folder=folder)

        kwargs = {
            "Key": "{}/{}".format(folder, filename),
            "Body": data,
            "Bucket": "acname",
            "ContentType": content_type,
        }
        self.amazon.client.put_object.assert_called_once_with(**kwargs)

    def test_find_folder(self):
        self.amazon.client = Mock()

        # Mock getting list of files from Amazon, first request returns a file, second request returns empty list
        self.amazon.client.list_objects = Mock(
            side_effect=[
                {
                    "Contents": [
                        {
                            "Key": "gridtest/abcd1234",
                            "LastModified": utcnow() - timedelta(minutes=30),
                            "Size": 500,
                            "ETag": "abcd1234",
                        }
                    ]
                },
                {"Contents": []},
            ]
        )

        folder = "gridtest"
        self.amazon.find(folder=folder)

        call_arg_list = [
            ({"Bucket": "acname", "Marker": "", "MaxKeys": 1000, "Prefix": "{}/".format(folder)},),
            ({"Bucket": "acname", "Marker": "gridtest/abcd1234", "MaxKeys": 1000, "Prefix": "{}/".format(folder)},),
        ]

        # We test the call_args_list as self.amazon.client.list_objects would have been called twice
        self.assertEqual(self.amazon.client.list_objects.call_count, 2)
        self.assertEqual(self.amazon.client.list_objects.call_args_list, call_arg_list)

    def test_guess_extension(self):
        self.assertEqual(".jpg", guess_media_extension("image/jpeg"))
        self.assertEqual(".png", guess_media_extension("image/png"))

        self.assertEqual(".mp3", guess_media_extension("audio/mp3"))
        self.assertEqual(".mp3", guess_media_extension("audio/mpeg"))
        self.assertEqual(".flac", guess_media_extension("audio/flac"))

        self.assertEqual(".mp4", guess_media_extension("video/mp4"))

        # leave empty when there is no extension
        self.assertEqual("", guess_media_extension("audio/foo"))

    def test_media_url_none_utf8(self):
        filename = "[DIARY NOTE] – Victory In The Pacific Day Commemoration - Thursday (1)"
        # automatic version is set on 15mins granularity.
        mins_granularity = int(int(time.strftime("%M")) / 4) * 4
        time_id = "%s%s" % (time.strftime("%Y%m%d%H%m"), mins_granularity)
        media_id = self.amazon.media_id(filename)
        self.assertEqual(
            "%s/%s" % (time_id, "DIARY NOTE - Victory In The Pacific Day Commemoration - Thursday (1)"), media_id
        )
        self.assertEqual(self.amazon.url_for_media(media_id), "https://acname.s3-us-east-1.amazonaws.com/%s" % media_id)
        sub = "test-sub"
        settings = {"AMAZON_S3_SUBFOLDER": sub, "MEDIA_PREFIX": "https://acname.s3-us-east-1.amazonaws.com/" + sub}
        with patch.dict(self.app.config, settings):
            media_id = self.amazon.media_id(filename)
            self.assertEqual(
                "%s/%s" % (time_id, "DIARY NOTE - Victory In The Pacific Day Commemoration - Thursday (1)"), media_id
            )
            path = "%s/%s" % (sub, media_id)
            self.assertEqual(self.amazon.url_for_media(media_id), "https://acname.s3-us-east-1.amazonaws.com/%s" % path)
            with patch.object(self.amazon, "client") as s3:
                self.amazon.get(media_id)
                self.assertTrue(s3.get_object.called)
                self.assertEqual(s3.get_object.call_args[1], dict(Bucket="acname", Key=path))

    def test_put_into_folder_none_utf8(self):
        data = b"test data"
        folder = "s3test"
        filename = "[DIARY NOTE] – Victory In The Pacific Day Commemoration - Thursday (1).pdf"
        content_type = "application/pdf"
        self.amazon.client.put_object = Mock()
        self.amazon._check_exists = Mock(return_value=False)

        self.amazon.put(data, filename, content_type, folder=folder, version=False)

        kwargs = {
            "Key": "{}/{}".format(folder, "DIARY NOTE - Victory In The Pacific Day Commemoration - Thursday (1).pdf"),
            "Body": data,
            "Bucket": "acname",
            "ContentType": content_type,
        }
        self.amazon.client.put_object.assert_called_once_with(**kwargs)

    def test_get_none_utf8(self):
        self.amazon.client.get_object = Mock()
        self.amazon.extract_metadata_from_headers = Mock(return_value={})

        self.amazon.get("[DIARY NOTE] – Victory In The Pacific Day Commemoration - Thursday (1).pdf")

        kwargs = {"Bucket": "acname", "Key": "DIARY NOTE - Victory In The Pacific Day Commemoration - Thursday (1).pdf"}
        self.amazon.client.get_object.assert_called_once_with(**kwargs)

    def test_mimetype_detect(self):
        # keep default mimetype
        content = b"bytes are here"
        filename = "extensionless"
        content_type = "text/css"
        folder = "f1"
        self.amazon.client.put_object = Mock()
        self.amazon._check_exists = Mock(return_value=False)
        self.amazon.media_id = Mock(return_value=filename)
        self.amazon.put(content, filename, content_type, folder=folder)
        self.amazon.client.put_object.assert_called_once_with(
            **{
                "Key": "{}/{}".format(folder, filename),
                "Body": content,
                "Bucket": "acname",
                "ContentType": content_type,
            }
        )

        # get mimetype from the filename
        content = b"bytes are here"
        filename = "styles.css"
        content_type = "application/pdf"
        folder = "f1"
        self.amazon.client.put_object = Mock()
        self.amazon._check_exists = Mock(return_value=False)
        self.amazon.media_id = Mock(return_value=filename)
        self.amazon.put(content, filename, content_type, folder=folder)
        self.amazon.client.put_object.assert_called_once_with(
            **{
                "Key": "{}/{}".format(folder, filename),
                "Body": b"bytes are here",
                "Bucket": "acname",
                "ContentType": "text/css",
            }
        )

        content = b"bytes are here"
        filename = "styles.JpG"
        content_type = "application/pdf"
        folder = "f1"
        self.amazon.client.put_object = Mock()
        self.amazon._check_exists = Mock(return_value=False)
        self.amazon.media_id = Mock(return_value=filename)
        self.amazon.put(content, filename, content_type, folder=folder)
        self.amazon.client.put_object.assert_called_once_with(
            **{
                "Key": "{}/{}".format(folder, filename),
                "Body": b"bytes are here",
                "Bucket": "acname",
                "ContentType": "image/jpeg",
            }
        )

        # get mimetype from the file
        fixtures_path = os.path.join(os.path.dirname(__file__), "fixtures")

        with open(os.path.join(fixtures_path, "file_example-jpg.jpg"), "rb") as content:
            filename = "extensionless"
            content_type = "dummy/text"
            folder = "f1"
            self.amazon.client.put_object = Mock()
            self.amazon._check_exists = Mock(return_value=False)
            self.amazon.media_id = Mock(return_value=filename)
            self.amazon.put(content, filename, content_type, folder=folder)
            self.amazon.client.put_object.assert_called_once_with(
                **{
                    "Key": "{}/{}".format(folder, filename),
                    "Body": content,
                    "Bucket": "acname",
                    "ContentType": "image/jpeg",
                }
            )

            with open(os.path.join(fixtures_path, "file_example-xls.xls"), "rb") as content:
                filename = "extensionless"
                content_type = "dummy/text"
                folder = "f1"
                self.amazon.client.put_object = Mock()
                self.amazon._check_exists = Mock(return_value=False)
                self.amazon.media_id = Mock(return_value=filename)
                self.amazon.put(content, filename, content_type, folder=folder)
                self.amazon.client.put_object.assert_called_once_with(
                    **{
                        "Key": "{}/{}".format(folder, filename),
                        "Body": content,
                        "Bucket": "acname",
                        "ContentType": "application/vnd.ms-excel",
                    }
                )

            with open(os.path.join(fixtures_path, "file_example-xlsx.xlsx"), "rb") as content:
                filename = "extensionless"
                content_type = "dummy/text"
                folder = "f1"
                self.amazon.client.put_object = Mock()
                self.amazon._check_exists = Mock(return_value=False)
                self.amazon.media_id = Mock(return_value=filename)
                self.amazon.put(content, filename, content_type, folder=folder)
                self.amazon.client.put_object.assert_called_once_with(
                    **{
                        "Key": "{}/{}".format(folder, filename),
                        "Body": content,
                        "Bucket": "acname",
                        "ContentType": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
                    }
                )

            with open(os.path.join(fixtures_path, "file_example-docx.docx"), "rb") as content:
                filename = "extensionless"
                content_type = "dummy/text"
                folder = "f1"
                self.amazon.client.put_object = Mock()
                self.amazon._check_exists = Mock(return_value=False)
                self.amazon.media_id = Mock(return_value=filename)
                self.amazon.put(content, filename, content_type, folder=folder)
                self.amazon.client.put_object.assert_called_once_with(
                    **{
                        "Key": "{}/{}".format(folder, filename),
                        "Body": content,
                        "Bucket": "acname",
                        "ContentType": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
                    }
                )



import logging
from flask import current_app as app
from flask_babel import lazy_gettext
from superdesk import get_resource_service

logger = logging.getLogger(__name__)


def get_default_content_template(item, **kwargs):
    if "dest_desk_id" in kwargs:
        desk = None
        desk_id = kwargs["dest_desk_id"]
    elif "desk" in kwargs:
        desk = kwargs["desk"]
        desk_id = kwargs["desk"]["_id"]
    elif "task" in item and "desk" in item["task"]:
        desk = None
        desk_id = item["task"].get("desk")
    else:
        logger.warning("Can't set default data, no desk identifier found")
        return

    if desk is None:
        desk = get_resource_service("desks").find_one(req=None, _id=desk_id)
    if not desk:
        logger.warning('Can\'t find desk with id "{desk_id}"'.format(desk_id=desk_id))
        return

    content_template_id = desk.get("default_content_template")
    if not content_template_id:
        logger.warning("No default content template set for {desk_name}".format(desk_name=desk.get("name", desk_id)))
        return
    content_template = get_resource_service("content_templates").find_one(req=None, _id=content_template_id)
    if not content_template:
        logger.warning(
            'Can\'t find content_template with id "{content_template_id}"'.format(
                content_template_id=content_template_id
            )
        )
        return

    return content_template


def set_default_template_metadata(item, **kwargs):

    fields_to_exclude = app.config.get("DEFAULT_TEMPLATE_METADATA_MACRO_EXCLUDE", {})
    fields_to_override = app.config.get("DEFAULT_TEMPLATE_METADATA_MACRO_OVERRIDE", {})

    """Replace some metadata from default content template"""

    content_template = get_default_content_template(item, **kwargs)
    if not content_template:
        return

    data = content_template["data"]

    vocabularies = get_resource_service("vocabularies").get(req=None, lookup={"field_type": {"$exists": True}})
    for vocabulary in vocabularies:
        fields_to_exclude.append(vocabulary["_id"])

    for key, value in data.items():
        if (not item.get(key) and key not in fields_to_exclude) or key in fields_to_override:
            item[key] = data.get(key)

    # subject contains remaining metadata to copy
    subject = data.setdefault("subject", [])

    # we first take out the metadata which we want to add, if any
    to_add = []
    for sub in subject:
        if sub.get("scheme") and not any(
            subjects.get("scheme") == sub.get("scheme") for subjects in item.get("subject", [])
        ):
            to_add.append(sub)

    # and now we add the new one
    item.setdefault("subject", []).extend([i for i in to_add if i.get("scheme") and to_add])

    if not item["subject"] and data.get("subject"):
        item["subject"] = data.get("subject")

    return item


name = "Set Default Template Metadata"
label = lazy_gettext("Set Default Template Metadata")
callback = set_default_template_metadata
access_type = "backend"
action_type = "direct"



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from content_api.items.service import ItemsService
from flask import g

from superdesk import get_resource_privileges
from superdesk.errors import SuperdeskApiError


class SearchService(ItemsService):
    """This class wraps the external Items API endpoint implemented by ItemsService but will allow the filtering by
    subscriber and the return of aggregates
    """

    # Usage is something like :-
    #  http://localhost:5000/api/search_capi?where={"headline":"muppets"}&start_date=1975-12-31
    #        &subscribers=558384d31d41c849d9614500
    #
    #    or for all subscribers
    #    http://localhost:5000/api/search_capi?where={"headline":"muppets"}&start_date=1975-12-31

    allowed_params = {
        "start_date",
        "end_date",
        "include_fields",
        "exclude_fields",
        "max_results",
        "page",
        "where",
        "version",
        "subscribers",
        "aggregations",
        "q",
        "default_operator",
        "sort",
        "filter",
        "service",
        "subject",
        "genre",
        "urgency",
        "priority",
        "type",
        "item_source",
    }

    excluded_fields_from_response = set()

    def _filter_empty_vals(self, data):
        """Filter out `None` values from a given dict."""
        return dict(filter(lambda x: x[1], data.items()))

    def _format_cv_item(self, item):
        return self._filter_empty_vals(
            {
                "qcode": item.get("code"),
                "name": item.get("name"),
            }
        )

    def _map_response(self, response):
        """
        Map the fields names that are changed as part of the ninjs formatting back to the internal values
        :param response:
        :return:
        """
        if not response:
            return None

        for item in response:
            self._map_item(item)

        return response

    def _map_item(self, item):
        """Maps the associations.
        :param dict item:
        :return:
        """
        if not item:
            return

        if item.get("service"):
            item["anpa_category"] = [self._format_cv_item(item) for item in (item.get("service") or [])]
            item.pop("service")

        if item.get("subject"):
            item["subject"] = [self._format_cv_item(item) for item in (item.get("subject") or [])]

        if item.get("genre"):
            item["genre"] = [self._format_cv_item(item) for item in (item.get("genre") or [])]

        if item.get("place"):
            item["place"] = [self._format_cv_item(item) for item in (item.get("place") or [])]

        if item.get("signal"):
            item["flags"] = {"marked_for_legal": True for signal in item.get("signal") if signal.get("code") == "cwarn"}

    def find_one(self, req, **lookup):
        self.check_get_access_privilege()
        response = super().find_one(req, **lookup)
        self._map_item(response)
        return response

    def get(self, req, lookup):
        # if there is a subscriber argumment map it into the lookup
        self.check_get_access_privilege()
        if req and req.args and req.args.get("subscribers"):
            lookup = {"subscribers": req.args.get("subscribers")}
            g.subscriber = req.args.get("subscribers")

        response = super().get(req, lookup)
        if response.count() > 0:
            return self._map_response(response)
        return response

    def _process_fetched_object(self, document):
        """Does some processing on the raw document fetched from database.

        It sets the item's `uri` field and removes all the fields added by the
        `Eve` framework that are not part of the NINJS standard (except for
        the HATEOAS `_links` object).
        It also sets the URLs for all externally referenced media content.

        :param dict document: MongoDB document to process
        """
        self._process_item_renditions(document)
        self._process_item_associations(document)
        self._map_associations(document)

    def _map_associations(self, item):
        """Map association response.

        :param dict item:
        """
        allowed_items = {}
        if item.get("associations"):
            for k, v in item.get("associations", {}).items():
                if v is None:
                    continue
                self._map_item(v)
                v["_id"] = v.get("guid", v.get("_id"))
                allowed_items[k] = v

        item["associations"] = allowed_items

    def check_get_access_privilege(self):
        """Checks if user is authorized to perform get operation on search api.

        If authorized then request is
        forwarded otherwise throws forbidden error.

        :raises: SuperdeskApiError.forbiddenError() if user is unauthorized to access the Legal Archive resources.
        """

        if not hasattr(g, "user"):
            return

        privileges = g.user.get("active_privileges", {})
        resource_privileges = get_resource_privileges(self.datasource).get("GET", None)
        if privileges.get(resource_privileges, 0) == 0:
            raise SuperdeskApiError.forbiddenError()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from decimal import Decimal
from . import unit_base
from flask_babel import lazy_gettext

CUBIC_METER_SYMBOL = "kg"


def convert(pounds, precision=1):
    """
    Converts from pounds to kg
    :param pounds: Pounds value in string
    :return: Kg value in string, and the symbol
    """
    lb_to_kg_rate = Decimal(0.453592)
    symbol = CUBIC_METER_SYMBOL
    pounds_list = pounds.split("-")
    kg_list = [unit_base.format_converted(Decimal(a) * lb_to_kg_rate, precision) for a in pounds_list]
    return "-".join(kg_list), symbol


def pounds_to_metric(item, **kwargs):
    """Converts pound values to metric"""

    regex = r"(\d+-?,?\.?\d*)((\s*)|(-))((lbs?)|([pP]ounds?))\b"
    return unit_base.do_conversion(item, convert, unit_base.format_output, regex, match_index=0, value_index=1)


name = "pounds_to_metric"
label = lazy_gettext("Weight pounds to metric")
callback = pounds_to_metric
access_type = "frontend"
action_type = "interactive"
group = lazy_gettext("temperature and weight")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import logging
import superdesk
from urllib.parse import urlparse
from io import BytesIO
from flask import current_app as app

from superdesk.ftp import ftp_connect
from superdesk.publish import register_transmitter
from superdesk.publish.publish_service import get_publish_service, PublishService
from superdesk.errors import PublishFtpError
from superdesk.media.renditions import get_rendition_file_name, get_renditions_spec

errors = [PublishFtpError.ftpError().get_error_description()]

logger = logging.getLogger(__name__)


def get_renditions_filter():
    renditions = set(get_renditions_spec(without_internal_renditions=True).keys())
    renditions.add("original")
    return renditions


class FTPPublishService(PublishService):
    """FTP Publish Service.

    It creates files on configured FTP server.

    :param string username: auth username
    :param string password: auth password
    :param path: server path
    :param passive: use passive mode (on by default)
    """

    NAME = "FTP"
    CONFIG = {"passive": True}

    def config_from_url(self, url):
        """Parse given url into ftp config. Used for tests.

        :param url: url in form `ftp://username:password@host:port/dir`
        """
        url_parts = urlparse(url)
        return {
            "username": url_parts.username,
            "password": url_parts.password,
            "host": url_parts.hostname,
            "path": url_parts.path.lstrip("/"),
        }

    def _get_published_item(self, queue_item):
        try:
            return json.loads(queue_item["formatted_item"])
        except json.JSONDecodeError as ex:
            return superdesk.get_resource_service("published").find_one(
                req=None,
                item_id=queue_item["item_id"],
                _current_version=queue_item["item_version"],
            )

    def _transmit(self, queue_item, subscriber):
        config = queue_item.get("destination", {}).get("config", {})

        try:
            with ftp_connect(config) as ftp:

                if config.get("push_associated", False):
                    # Set the working directory for the associated files
                    if "associated_path" in config and config.get("associated_path"):
                        ftp.cwd("/" + config.get("associated_path", "").lstrip("/"))

                    item = self._get_published_item(queue_item)
                    if item:
                        self._copy_published_media_files(item, ftp)

                    # If the directory was changed to push associated files change it back
                    if "associated_path" in config and config.get("associated_path"):
                        ftp.cwd("/" + config.get("path").lstrip("/"))

                filename = get_publish_service().get_filename(queue_item)
                b = BytesIO(queue_item.get("encoded_item", queue_item.get("formatted_item").encode("UTF-8")))
                ftp.storbinary("STOR " + filename, b)
        except PublishFtpError:
            raise
        except Exception as ex:
            raise PublishFtpError.ftpError(ex, queue_item.get("destination"))

    def _copy_published_media_files(self, item, ftp):
        renditions_filter = get_renditions_filter()

        def parse_media(item):
            media = {}
            renditions = item.get("renditions", {})
            for key, rendition in renditions.items():
                if key not in renditions_filter:
                    continue
                if not rendition.get("media"):
                    logger.warn("media missing on rendition %s for item %s", key, item["guid"])
                    continue
                rendition.pop("href", None)
                rendition.setdefault("mimetype", rendition.get("original", {}).get("mimetype", item.get("mimetype")))
                media[rendition["media"]] = rendition
            return media

        media = {}
        media.update(parse_media(item))

        for assoc in item.get("associations", {}).values():
            if assoc is None:
                continue
            media.update(parse_media(assoc))
            for assoc2 in assoc.get("associations", {}).values():
                if assoc2 is None:
                    continue
                media.update(parse_media(assoc2))

        # Retrieve the list of files that currently exist in the FTP server
        remote_items = []
        ftp.retrlines("LIST", remote_items.append)

        for media_id, rendition in media.items():
            if not self._media_exists(rendition, remote_items):
                binary = app.media.get(media_id, resource=rendition.get("resource", "upload"))
                self._transmit_media(binary, rendition, ftp)

    def _media_exists(self, rendition, items):
        for file in items:
            if get_rendition_file_name(rendition) in file:
                return True
        return False

    def _transmit_media(self, binary, rendition, ftp):
        ftp.storbinary("STOR " + get_rendition_file_name(rendition), binary)


register_transmitter("ftp", FTPPublishService(), errors)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from unittest import mock

from superdesk.activity import ActivityService, ACTIVITY_ERROR, get_recipients
from superdesk.publish import init_app
from superdesk.tests import TestCase


def mock_get_resource_service(resource_name):
    return MockPreferenceService()


class MockPreferenceService:
    def email_notification_is_enabled(self, preferences=None):
        send_email = preferences.get("email:notification", {}) if isinstance(preferences, dict) else {}

        return send_email and send_email.get("enabled", False)


class ActivityTestCase(TestCase):
    def setUp(self):
        init_app(self.app)

    def test_if_user_is_in_recipients(self):
        activity = {"recipients": [{"user_id": "1", "read": True}, {"user_id": "2", "read": False}]}

        self.assertTrue(ActivityService().is_recipient(activity, "1"))
        self.assertTrue(ActivityService().is_recipient(activity, "2"))
        self.assertFalse(ActivityService().is_recipient(activity, "3"))

    def test_is_read(self):
        activity = {"recipients": [{"user_id": "1", "read": True}, {"user_id": "2", "read": False}]}

        self.assertTrue(ActivityService().is_read(activity, "1"))
        self.assertFalse(ActivityService().is_read(activity, "2"))
        self.assertFalse(ActivityService().is_read(activity, "3"))

    @mock.patch("superdesk.activity.get_resource_service", mock_get_resource_service)
    def test_get_recipients_filters_out_users_not_activated(self):
        users = [
            {
                "email": "test_1@test.com",
                "needs_activation": False,
                "is_enabled": True,
                "is_active": True,
                "user_preferences": {"email:notification": {"enabled": True}},
            },
            {
                "email": "test_2@test.com",
                "needs_activation": True,
                "is_enabled": True,
                "is_active": True,
                "user_preferences": {"email:notification": {"enabled": True}},
            },
        ]

        recipients = get_recipients(user_list=users, activity_name=ACTIVITY_ERROR)
        self.assertEqual(len(recipients), 1)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.search_providers import allowed_search_providers
from superdesk.resource import Resource
from superdesk.utils import required_string


class SearchProviderResource(Resource):
    schema = {
        "search_provider": {"type": "string", "required": True, "allowed": allowed_search_providers},
        "source": required_string,
        "name": {"type": "string"},
        "is_closed": {"type": "boolean", "default": False},
        "is_default": {"type": "boolean", "default": False},
        "advanced_search": {
            "type": "boolean",
            "default": False,
        },
        "last_item_update": {"type": "datetime"},
        "config": {"type": "dict"},
    }

    etag_ignore_fields = ["last_item_update"]

    resource_methods = ["GET", "POST"]
    item_methods = ["GET", "PATCH", "DELETE"]

    privileges = {"POST": "search_providers", "PATCH": "search_providers", "DELETE": "search_providers"}



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : pablopunk
# Creation: 2020-09-09 14:08

from superdesk.commands.data_updates import BaseDataUpdate
from eve.utils import config


class DataUpdate(BaseDataUpdate):

    resource = "vocabularies"

    def forwards(self, mongodb_collection, mongodb_database):
        for vocabulary in mongodb_collection.find({"_id": "usageterms"}):
            if "schema_field" not in vocabulary:
                mongodb_collection.update(
                    {"_id": vocabulary.get(config.ID_FIELD)}, {"$set": {"schema_field": "usageterms"}}
                )

    def backwards(self, mongodb_collection, mongodb_database):
        pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from unittest import mock
from unittest.mock import MagicMock
from flask import Flask
from werkzeug.datastructures import MultiDict
from content_api.tests import ApiTestCase


class SearchServiceTestCase(ApiTestCase):
    """Base class for the `items` service tests."""

    def setUp(self):
        self.app = Flask(__name__)
        self.ctx = self.app.test_request_context("/")
        self.ctx.push()

    def tearDown(self):
        self.ctx.pop()

    def _get_target_class(self):
        """Return the class under test.

        Make the test fail immediately if the class cannot be imported.
        """
        try:
            from content_api.search import SearchService
        except ImportError:
            self.fail("Could not import class under test (SearchService).")
        else:
            return SearchService

    def _make_one(self, *args, **kwargs):
        """Create a new instance of the class under test."""
        return self._get_target_class()(*args, **kwargs)


fake_super_get = MagicMock(name="fake super().get")


@mock.patch("content_api.items.service.BaseService.get", fake_super_get)
class MapResponseTestCast(SearchServiceTestCase):
    def test_format_cv(self):
        instance = self._make_one()
        item = instance._format_cv_item({"code": "x", "name": "test"})
        self.assertEqual(item["qcode"], "x")

    def test_map_item(self):
        instance = self._make_one()
        item = {
            "service": [{"code": "x", "name": "foo"}],
            "genre": [{"code": "y", "name": "bar"}],
            "subject": [{"code": "a", "name": "xyz"}],
            "place": [{"code": "a", "name": "xyz"}],
            "headline": "headline",
        }

        instance._map_item(item)

        expected = {
            "anpa_category": [{"qcode": "x", "name": "foo"}],
            "genre": [{"qcode": "y", "name": "bar"}],
            "subject": [{"qcode": "a", "name": "xyz"}],
            "place": [{"qcode": "a", "name": "xyz"}],
            "headline": "headline",
        }

        self.assertEqual(item, expected)

    def test_aggregations(self):
        fake_request = MagicMock()
        fake_request.args = MultiDict([("aggregations", 1)])
        lookup = {}
        fake_response = MagicMock()
        fake_response.count.return_value = 1
        fake_super_get.return_value = fake_response

        instance = self._make_one()
        instance.get(fake_request, lookup)

        self.assertTrue(fake_super_get.called)
        args, _ = fake_super_get.call_args

        self.assertGreater(len(args), 0)
        self.assertEqual(args[0].args["aggregations"], 1)

        fake_request = MagicMock()
        fake_request.args = MultiDict([("aggregations", 0)])
        lookup = {}

        instance = self._make_one()
        instance.get(fake_request, lookup)

        self.assertTrue(fake_super_get.called)
        args, _ = fake_super_get.call_args

        self.assertGreater(len(args), 0)
        self.assertEqual(args[0].args["aggregations"], 0)



from superdesk.resource import Resource


class ExportResource(Resource):
    """Export schema"""

    endpoint_name = "export"
    resource_methods = ["POST"]
    item_methods = []
    resource_title = endpoint_name
    schema = {
        "item_ids": {"type": "list", "required": True},
        "format_type": {"type": "string", "required": True},
        "validate": {"type": "boolean", "required": False},
        "failures": {
            "type": "integer",
            "readonly": True,
        },
        "url": {
            "type": "string",
            "nullable": True,
        },
    }
    privileges = {"POST": "archive"}



from .clean_images import CleanImages  # noqa
from .rebuild_elastic_index import RebuildElasticIndex  # noqa
from .index_from_mongo import IndexFromMongo  # noqa
from .run_macro import RunMacro  # noqa
from .data_updates import *  # noqa
from .delete_archived_document import *  # noqa
from .update_archived_document import *  # noqa
from .remove_exported_files import RemoveExportedFiles  # noqa
from .flush_elastic_index import FlushElasticIndex  # noqa
from .generate_vocabularies import GenerateVocabularies  # noqa
from . import schema  # noqa


from superdesk.celery_app import celery


@celery.task()
def temp_file_expiry():
    RemoveExportedFiles()



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.io.feed_parsers.nitf import NITFFeedParser
from superdesk.io.registry import register_feed_parser
from superdesk.metadata.item import FORMAT
from superdesk.io.iptc import subject_codes
from flask import current_app as app
from apps.archive.common import format_dateline_to_locmmmddsrc
from superdesk.utc import get_date
import logging

logger = logging.getLogger("EFEFeedParser")


class EFEFeedParser(NITFFeedParser):
    """
    Feed parser for the NITF feed from Agencia EFE
    """

    NAME = "efe_nitf"

    label = "EFE NITF"

    def __init__(self):
        self.MAPPING = {
            "guid": {"xpath": "head/docdata/doc-id/@id-string", "default": None},
            "uri": {"xpath": "head/docdata/doc-id/@id-string", "default": None},
            "urgency": {
                "xpath": "head/docdata/urgency/@ed-urg",
                "default_attr": 5,
                "filter": int,
            },
            "pubstatus": {
                "xpath": "head/docdata/@management-status",
                "default_attr": "usable",
            },
            "firstcreated": {
                "xpath": "head/docdata/date.issue",
                "filter": self.get_norm_datetime,
            },
            "versioncreated": {
                "xpath": "head/docdata/date.issue",
                "filter": self.get_norm_datetime,
            },
            "expiry": {
                "xpath": "head/docdata/date.expire",
                "filter": self.get_norm_datetime,
            },
            "subject": self.get_subjects,
            "body_html": self.get_content,
            FORMAT: self.get_format,
            "place": self.get_place,
            "keywords": {
                "xpath": "head/docdata",
                "filter": self.get_keywords,
            },
            "slugline": {
                "xpath": "head/docdata",
                "filter": self.get_slugline,
            },
            "genre": self.get_genre,
            "ednote": "head/docdata/ed-msg/@info",
            "headline": self.get_headline,
            "abstract": self.get_abstract,
        }

        super().__init__()
        self.default_mapping = {}

    def parse(self, xml, provider=None):
        item = super().parse(xml, provider)
        self.derive_dateline(item)
        return item

    def derive_dateline(self, item):
        """
        Attempt to derive a dateline using the place, only if there is exactly one match on the city can we be sure we
        have the correct country.
        :param item:
        :return:
        """
        try:
            if len(item.get("place", [])) == 1:
                cities = app.locators.find_cities()
                city = item.get("place", "")[0].get("name", "")
                if city:
                    located = [c for c in cities if c["city"].lower() == city.lower()]
                    if len(located) == 1:
                        item.setdefault("dateline", {})
                        item["dateline"]["located"] = located[0]
                        item["dateline"]["source"] = item.get("original_source", "EFE")
                        item["dateline"]["text"] = format_dateline_to_locmmmddsrc(
                            item["dateline"]["located"],
                            get_date(item["firstcreated"]),
                            source=item.get("original_source", "EFE"),
                        )
        except Exception as ex:
            logging.exception("EFE dateline extraction exception {}".format(ex))
        finally:
            item.pop("place", None)

    def get_slugline(self, docdata):
        """
        Get the keywords and put the first one into the slugline
        :param docdata:
        :return:
        """
        keywords = self.get_keywords(docdata)
        return keywords[0] if len(keywords) > 0 else None

    def get_subjects(self, tree):
        """Finds all the IPTC subject tags in the passed tree and returns the parsed subjects.

        All entries will have both the name and qcode populated.

        :param tree:
        :return: a list of subject dictionaries
        """
        subjects = []
        qcodes = []  # we check qcodes to avoid duplicates
        for elem in tree.findall('head/tobject/tobject.subject[@tobject.subject.ipr="IPTC"]'):
            qcode = elem.get("tobject.subject.refnum")
            if qcode in qcodes:
                # we ignore duplicates
                continue
            else:
                qcodes.append(qcode)

            # if the subject_fields are not specified.
            if not any(c["qcode"] == qcode for c in subjects) and subject_codes.get(qcode):
                subjects.append({"name": subject_codes[qcode], "qcode": qcode})
        return subjects


register_feed_parser(EFEFeedParser.NAME, EFEFeedParser())
